#!/usr/bin/env python
import argparse
import logging
import os.path

from time import sleep
from urllib.parse import parse_qs, urlparse

import tempita

from bs4 import BeautifulSoup as bs4

import ptpapi


def ellipsize(string, length):
    if len(string) > length:
        return string[: length - 3] + "..."
    return string


def do_inbox(api, args):
    page = args.page
    user = api.current_user()
    if args.mark_all_read:
        print("Clearing out {0} messages".format(user.get_num_messages()))
        while user.new_messages > 0:
            for msg in api.current_user().inbox(page=page):
                if msg["Unread"] is False:
                    continue
                user.inbox_conv(msg["ID"])
            page += 1
    elif args.conversation:
        conv = user.inbox_conv(args.conversation)
        print(conv["Subject"])
        for msg in conv["Message"]:
            print("{0} - {1}\n".format(msg["User"], msg["Time"]))
            print(msg["Text"])
            print("----------------------------")
    elif args.mark_read:
        for conv in args.mark_read:
            user.inbox_conv(conv)
    else:
        msgs = list(user.inbox(page=page))
        print("ID" + " " * 8 + "Subject" + " " * 25 + "Sender" + " " * 9)
        print("-" * 55)
        for msg in msgs:
            if args.unread and msg["Unread"] is False:
                continue
            if args.user is not None and msg["Sender"] != args.user:
                continue
            print(
                "{0: <10}{1: <32}{2: <15}".format(
                    msg["ID"],
                    ellipsize(msg["Subject"].decode("utf-8"), 31),
                    ellipsize(msg["Sender"], 15),
                )
            )


def parse_terms(termlist):
    """Takes an array of terms, and sorts them out into 4 categories:
    * torrent URLs
    * movie URLs
    * targets (where to perform the search e.g. collages or bookmarks)
    * all other search parameters
    """
    torrents = []
    movies = []
    terms = {}
    target = "torrents"

    for arg in termlist:
        url = urlparse(arg)
        url_args = parse_qs(url.query)
        if url.path == "/collages.php":
            target = "collage"
            terms = url_args
        elif url.path == "/artist.php":
            target = "artist"
            terms = url_args
        elif url.path == "/torrents.php":
            if "torrentid" in url_args:
                torrents.append(ptpapi.Torrent(url_args["torrentid"][0]))
            elif "id" in url_args:
                if "action" in url_args and url_args["action"][0] == "download":
                    torrents.append(ptpapi.Torrent(url_args["id"][0]))
                else:
                    movies.append(ptpapi.Movie(url_args["id"][0]))
            else:
                terms = url_args
        else:
            term = arg.partition("=")
            if not term[2]:
                if term[0] == "bookmarks":
                    target = "bookmarks"
                else:
                    terms["searchstr"] = term[0]
            else:
                # Provide aliases for commonly used terms
                term_map = {
                    "taglist": ["genre", "genres", "tags"],
                    "searchstr": ["name", "title"],
                }
                for key, value in term_map.items():
                    if term[0] in value:
                        term = (key, term[1], term[2])
                terms[term[0]] = term[2]
    return (target, movies, torrents, terms)


def get_pages(target, terms):
    if target == "torrents":
        return ptpapi.util.find_page_range(
            ptpapi.session.session.base_get("torrents.php", params=terms).content
        )
    return None


def do_search(api, args):
    (target, movies, torrents, terms) = parse_terms(args.search_terms)
    if args.all:
        args.pages = get_pages(target, terms)
        logger = logging.getLogger(__name__)
        logger.debug("Auto-detected maximum page as %s")
    if "page" not in terms:
        terms["page"] = "1"
    else:
        page = terms["page"][0]
        terms["page"] = page
    for _ in range(args.pages):
        search_page(api, args, target, movies, torrents, terms.copy())
        terms["page"] = str(int(terms["page"]) + 1)


def search_page(api, args, target, movies, torrents, terms):
    logger = logging.getLogger(__name__)
    if args.movie_format == "":
        movie_template = None  # Just to make linting happy
    elif args.movie_format is not None:
        movie_template = tempita.Template(args.movie_format)
    else:
        movie_template = tempita.Template(
            "{{Title}} ({{Year}}) - {{if Directors}}{{','.join([d['Name'].strip() for d in Directors])}} -{{endif}} "
            "[{{'/'.join(Tags)}}] - [PTP {{GroupId}}{{if ImdbId}}, IMDB tt{{ImdbId}}{{endif}}]"
        )
    if args.torrent_format == "":
        torrent_template = None
    elif args.torrent_format is not None:
        torrent_template = tempita.Template(args.torrent_format)
    else:
        torrent_template = tempita.Template(
            "{{if GoldenPopcorn}}\u2606{{else}}-{{endif}} {{Codec}}/{{Container}}/{{Source}}/{{Resolution}}"
            " - {{ReleaseName}} - {{Snatched}}/{{Seeders}}/{{Leechers}}"
        )

    # If we haven't found any URL-looking things
    if not movies and not torrents:
        logger.debug('Attempting to search target "%s" with terms %s', target, terms)
        if target == "torrents":
            movies = api.search(terms)
            # Check to see if we should scrape the cover view data to save calls
            wanted_fields = set(
                [l[2].split("|")[0] for l in movie_template._parsed if l[0] == "expr"]
            )
            if len(wanted_fields & set(api.search_coverview_fields)):
                for movie in api.search_coverview(terms):
                    for ret_movie in movies:
                        if movie["GroupId"] == ret_movie["GroupId"]:
                            ret_movie.update(movie)
        elif target == "bookmarks":
            movies = api.current_user().bookmarks(search_terms=terms)
        elif target == "collage":
            movies = api.collage(terms["id"], terms)
        elif target == "artist":
            movies = api.artist(terms["id"], terms)
        movies = movies[: args.limit]

    if args.download:
        for movie in movies[: args.limit]:
            if movie_template:
                print(movie_template.substitute(movie))
            match = movie.best_match(args.filter)
            if match:
                if torrent_template:
                    print(torrent_template.substitute(match))
                if not args.dry_run:
                    match.download_to_dir(args.output_directory)
                else:
                    logger.info("Dry-run, not downloading %s", match)
            else:
                logger.info(
                    "No match found for for movie %s (%s)",
                    movie["Title"],
                    movie["Year"],
                )
        for torrent in torrents:
            if args.download and not args.dry_run:
                if torrent_template:
                    print(torrent_template.substitute(torrent))
                torrent.download_to_dir(args.output_directory)
            elif args.dry_run:
                logger.info("Dry-run, not downloading %s", torrent)
    else:
        for movie in movies[: args.limit]:
            if movie_template:
                print(movie_template.substitute(movie))
            for torrent in movie["Torrents"]:
                if torrent_template:
                    print(torrent_template.substitute(torrent))
        for torrent in torrents:
            if torrent_template:
                print(torrent_template.substitute(torrent))


def do_raw(_, args):
    """Given a URL, download the raw HTML to the current directory"""
    for url_str in args.url:
        url = urlparse(url_str)
        data = ptpapi.session.session.base_get("?".join([url.path, url.query])).content
        if args.output:
            if args.output == "-":
                print(data.decode(), end="")
                return
            else:
                file_out = args.output
        else:
            file_out = os.path.basename(url.path)
        with open(file_out, "wb") as fileh:
            fileh.write(data)


def do_log(api, args):
    interval = 30.0
    lastmsg = None
    while True:
        printmsg = False
        msgs = api.log()
        # We actually want it 'reversed' by default, with the newest at the bottom
        if not args.reverse:
            msgs.reverse()
        for time, msg in msgs:
            if lastmsg is None or printmsg:
                print(time, "-", msg)
                lastmsg = msg
            if lastmsg == msg:
                printmsg = True
        if args.follow:
            sleep(interval)
        else:
            break


def do_fields(api, args):
    print("Movie:")
    m = ptpapi.Movie(ID=1)
    for values in m.key_finder.values():
        for val in values:
            print(f"- {val}")
    print("Torrent:")
    t = ptpapi.Torrent(ID=1)
    for values in t.key_finder.values():
        for val in values:
            print(f"- {val}")


def do_search_fields(api, args):
    soup = bs4(
        ptpapi.session.session.base_get(
            "torrents.php", params={"action": "advanced", "json": "0"}
        ).content,
        "html.parser",
    )
    for e in soup.find(id="filter_torrents_form")("input"):
        if (
            e["type"] in ["submit", "button"]
            or e["name"].startswith("filter_cat")
            or e["name"].startswith("tags_type")
            or e["name"].startswith("country_type")
            or e["name"] == "action"
        ):
            continue
        name = e["name"]
        if "placeholder" in e.attrs.keys():
            name += " - " + e["placeholder"]
        if "title" in e.attrs.keys():
            name += " - " + e["title"]
        print(name)


def do_userstats(api, args):
    if args.user_id:
        user = ptpapi.User(args.user_id)
    else:
        user = api.current_user()
    if args.hummingbird:
        # e.g. '[ Example ] :: [ Power User ] :: [ Uploaded: 10.241 TiB | Downloaded: 1.448 TiB | Points: 79,76g2,506 | Ratio: 2.58 ] :: [ https://passthepopcorn.me/user.php?id=XXXXX ]'
        stats = user.stats()
        stats["Id"] = user.ID
        print(
            "[ {{Username}} ] :: [ {Class} ] :: [ Uploaded: {Uploaded} | Downloaded: {Downloaded} | Points: {Points} | Ratio: {Ratio} ] :: [ https://passthepopcorn.me/user.php?id={Id} ]".format(
                **stats
            )
        )
    else:
        for stat, value in user.stats().items():
            print(stat + ": " + value)


def do_archive(api, args):
    r = ptpapi.session.session.base_get(
        "archive.php",
        params={
            "action": "fetch",
            "MaxStalled": 0,
            "ContainerName": ptpapi.config.config.get("PTP", "archiveContainerName"),
            "ContainerSize": ptpapi.config.config.get("PTP", "archiveContainerSize"),
        },
    )
    r.raise_for_status()
    data = r.json()
    ptpapi.Torrent(ID=data["TorrentID"]).download_to_dir(
        params={"ArchiveID": data["ArchiveID"]}
    )
    if args.download_incomplete:
        for _id, i_data in data["IncompleteTransactions"].items():
            if i_data["InfoHash"] is not None:
                ptpapi.Torrent(ID=i_data["TorrentID"]).download_to_dir(
                    params={"ArchiveID": data["ArchiveID"]}
                )


def add_verbosity_args(parser):
    """Helper function to improve DRY"""
    parser.add_argument(
        "--debug",
        help="Print lots of debugging statements",
        action="store_const",
        dest="loglevel",
        const=logging.DEBUG,
        default=logging.WARNING,
    )
    parser.add_argument(
        "-v",
        "--verbose",
        help="Be verbose",
        action="store_const",
        dest="loglevel",
        const=logging.INFO,
    )
    parser.add_argument(
        "-q",
        "--quiet",
        help="Hide most messages",
        action="store_const",
        dest="loglevel",
        const=logging.CRITICAL,
    )


def main():
    logger = logging.getLogger(__name__)
    parser = argparse.ArgumentParser(
        description="Extensible command line utility for PTP"
    )
    parser.set_defaults(func=None)
    add_verbosity_args(parser)
    subparsers = parser.add_subparsers()

    # Search & download
    search_parent = argparse.ArgumentParser()
    add_verbosity_args(search_parent)
    search_parent.add_argument(
        "search_terms",
        help="""A list of terms in [field]=[text] format.
        If the '=' is omitted, the field is assumed to be 'name'.""",
        nargs="+",
        metavar="term",
    )
    search_parent.add_argument(
        "-n",
        "--dry-run",
        help="Don't actually download any torrents",
        action="store_true",
    )
    search_parent.add_argument(
        "-l", "--limit", help="Limit search results to N movies", default=100, type=int
    )
    search_parent.add_argument(
        "-f",
        "--filter",
        help="Define a filter to download movies with",
        default=ptpapi.config.config.get("Main", "filter"),
    )
    search_parent.add_argument(
        "-m", "--movie-format", help="Set the output for movies", default=None
    )
    search_parent.add_argument(
        "-t", "--torrent-format", help="Set the output for torrents", default=None
    )
    search_parent.add_argument(
        "-o",
        "--output-directory",
        help="Location for any downloaded files",
        default=None,
    )
    search_parent.add_argument(
        "-p", "--pages", help="The number of pages to download", default=1, type=int
    )
    search_parent.add_argument(
        "-a", "--all", help="Return all search results", action="store_true"
    )

    # Search
    search_parser = subparsers.add_parser(
        "search",
        help="Search for or download movies",
        add_help=False,
        parents=[search_parent],
    )
    search_parser.add_argument(
        "-d", "--download", help="Download any movies found", action="store_true"
    )
    search_parser.set_defaults(func=do_search)

    # Download
    download_parser = subparsers.add_parser(
        "download",
        help="An alias for `search -d`",
        add_help=False,
        parents=[search_parent],
    )
    download_parser.add_argument(
        "-d",
        "--download",
        help="Download any movies found",
        action="store_true",
        default=True,
    )
    download_parser.set_defaults(func=do_search)

    # Archive
    archive_parser = subparsers.add_parser(
        "archive", help="Commands related to the archive project."
    )
    archive_parser.add_argument(
        "--download-incomplete",
        help="Also download any incomplete transactions",
        action="store_true",
    )
    archive_parser.set_defaults(func=do_archive)

    # Inbox
    inbox_parser = subparsers.add_parser("inbox", help="Reads messages in your inbox")
    add_verbosity_args(inbox_parser)
    inbox_parser.add_argument(
        "-u", "--unread", help="Only show unread messages", action="store_true"
    )
    inbox_parser.add_argument(
        "-m",
        "--mark-read",
        help="Mark messages as read",
        type=lambda s: [int(n) for n in s.split(",")],
    )
    inbox_parser.add_argument(
        "--mark-all-read",
        help="Scan and mark all messages as read. "
        "WARNING: If new messages arrive while this is running, the script can get caught in a loop until it reaches the end of the inbox's pages",
        action="store_true",
    )
    inbox_parser.add_argument("--user", help="Filter messages by the sender")
    inbox_parser.add_argument(
        "-c",
        "--conversation",
        help="Get the messages of a specific conversation",
        type=int,
    )
    inbox_parser.add_argument(
        "-p", "--page", help="Start at a certain page", type=int, default=1
    )
    inbox_parser.set_defaults(func=do_inbox)

    raw_parser = subparsers.add_parser("raw", help="Fetch the raw HTML of pages")
    add_verbosity_args(raw_parser)
    raw_parser.add_argument("url", help="A list of urls to download", nargs="+")
    raw_parser.add_argument(
        "-o",
        "--output",
        help="Set output file (or - for stdout)",
    )
    raw_parser.set_defaults(func=do_raw)

    # User stats
    userstats_parser = subparsers.add_parser(
        "userstats", help="Gather users' stats from profile pages"
    )
    add_verbosity_args(userstats_parser)
    userstats_parser.add_argument(
        "-i", "--user-id", help="The user to look at", nargs="?", default=None
    )
    userstats_parser.add_argument(
        "--hummingbird", help="Imitate Hummingbird's format", action="store_true"
    )
    userstats_parser.set_defaults(func=do_userstats)

    # Fields
    field_parser = subparsers.add_parser(
        "fields", help="List the fields available for each PTPAPI resource"
    )
    add_verbosity_args(field_parser)
    field_parser.set_defaults(func=do_fields)

    search_field_parser = subparsers.add_parser(
        "search-fields", help="List the fields available when searching"
    )
    add_verbosity_args(search_field_parser)
    search_field_parser.set_defaults(func=do_search_fields)

    log_parser = subparsers.add_parser("log", help="Show the log of recent events")
    add_verbosity_args(log_parser)
    log_parser.add_argument(
        "-r", "--reverse", help="Sort in reverse", action="store_true"
    )
    log_parser.add_argument(
        "-f", "--follow", help="Print new entries as they appear", action="store_true"
    )
    log_parser.set_defaults(func=do_log)

    args = parser.parse_args()

    logging.basicConfig(level=args.loglevel)

    api = ptpapi.login()

    if args.func is None:
        parser.print_help()
        return
    args.func(api, args)
    logger.debug(
        "Total session tokens consumed: %s", ptpapi.session.session.consumed_tokens
    )
    logger.debug("Exiting...")


if __name__ == "__main__":
    main()



import os
import re

import humanize

from ptpapi.config import config
from ptpapi.session import TokenSession
from ptpapi.sites.base import BaseSiteAPI


class KGAPI(BaseSiteAPI):
    Name = "KG"

    def __init__(self):
        self.baseURL = "https://karagarga.in"
        self.session = TokenSession(3, 0.5)
        self.session.headers.update({"User-Agent": "Wget/1.13.4"})
        super().__init__()

    def login(self, username=None, password=None, passkey=None):
        password = password or config.get("KG", "password")
        username = username or config.get("KG", "username")
        response = self.session.post(
            self.baseURL + "/takelogin.php",
            data={"username": username, "password": password},
        ).text
        if response.find('action="takelogin.php"') != -1:
            raise KGAPIException("Failed to log in")

    def search(self, search_args):
        search_string = "&".join(
            ["%s=%s" % (key, value) for (key, value) in search_args.items()]
        )
        soup = self._httpRequest("/browse.php?%s" % search_string)
        return self.getTorrentListInfo(soup)

    def getTorrentListInfo(self, soup):
        if not soup.find("table", id="browse"):
            return []
        retArray = []
        for row in soup.find("table", id="browse").find_all("tr")[1:]:
            if "id" in row.find_all("td")[0].attrs:
                # Only rows used for displaying extra info have IDs; we don't want them
                continue
            cells = row.find_all("td")
            infoDict = {
                "Title": cells[1].b.get_text(),
                "Year": cells[3].get_text(),
                "Seeders": cells[12].get_text(),
                "Leechers": cells[13].get_text(),
                "BinaryHumanSize": re.sub(
                    r"([a-zA-Z])B", r" \1iB", cells[10].get_text()
                ).replace("k", "K"),
            }
            infoDict["ID"] = re.search(r"\d+", cells[1].a["href"]).group(0)
            retArray.append(infoDict)
        return retArray

    def download(self, ID):
        r = self.session.get(self.baseURL + "/down.php/%s/file.torrent" % ID)
        downloadName = re.search(
            r'filename="(.*)"', r.headers["Content-Disposition"]
        ).group(1)
        return (downloadName, r.content)

    def download_to_file(self, ID, dest=None):
        r = self.session.get(self.baseURL + "/down.php/%s/file.torrent" % ID)
        r.raise_for_status()
        if not dest:
            name = (
                re.search(r'filename="(.*)"', r.headers["Content-Disposition"])
                .group(1)
                .replace("/", "_")
            )
            dest = os.path.join(config.get("Main", "downloadDirectory"), name)
        with open(dest, "wb") as fh:
            fh.write(r.content)

    def find_ptp_movie(self, movie):
        return self.search({"search_type": "imdb", "search": movie["ImdbId"]})

    def bytes_to_site_size(self, byte_num):
        humanized = humanize.naturalsize(byte_num, format="%.2f", binary=True)
        if "MiB" in humanized or "KiB" in humanized:
            humanized = humanize.naturalsize(byte_num, format="%d", binary=True)
        return humanized


class KGAPIException(Exception):
    pass


if __name__ == "__main__":
    kg = KGAPI()
    print(kg.search({"search_type": "imdb", "search": "0207295"}))



#!/usr/bin/env python
"""Reseed a torrent from PTP, given a path"""
import argparse
import logging
import os
import os.path
import sys

from pathlib import Path
from time import sleep, time
from typing import Optional
from urllib.parse import parse_qs, urlparse
from xmlrpc import client as xmlrpc_client

import bencode
import bencodepy
import libtc
import pyrosimple

from pyrosimple.util import metafile, rpc

import ptpapi


class Match:
    """A tiny class to make matching easier

    Could be expanded to introduce a confidence indicator

    ID is an integer-as-a-string, and path is filepath"""

    # pylint: disable=too-few-public-methods
    def __init__(
        self,
        ID: Optional[str] = None,
        path: Optional[os.PathLike] = None,
        matched_files: Optional[dict] = None,
    ):
        """A defined match"""
        self.ID = ID
        self.path = path
        if matched_files is None:
            matched_files = {}
        self.matched_files = matched_files

    def __nonzero__(self):
        if self.ID is not None and self.path is not None:
            return True
        return False

    def __bool__(self):
        return self.__nonzero__()

    def __str__(self):
        return "<Match {0}:{1}>".format(self.ID, self.path)


def match_by_torrent(torrent, filepath: str) -> Match:
    """Attempt matching against a torrent ID"""
    logger = logging.getLogger(__name__)
    logger.info(
        "Attempting to match against torrent {0} ({1})".format(
            torrent.ID, torrent["ReleaseName"]
        )
    )

    if isinstance(filepath, bytes):
        filepath = filepath.decode("utf-8")
    path1 = os.path.abspath(filepath)
    path1_files = {}
    if os.path.isdir(path1):
        for root, _, filenames in os.walk(path1, followlinks=True):
            for filename in filenames:
                realpath = os.path.join(root, filename).replace(
                    os.path.dirname(path1) + os.sep, ""
                )
                path1_files[realpath] = os.path.getsize(os.path.join(root, filename))
    elif os.path.isfile(path1):
        path1_files[os.path.basename(path1)] = os.path.getsize(path1)

    path2_files = dict((f, int(s)) for f, s in torrent["Filelist"].items())

    if len(path1_files) < len(path2_files):
        logger.debug(
            "Too little files to match torrent ({0} locally, {1} in torrent)".format(
                len(path1_files), len(path2_files)
            )
        )
        return Match(None)

    matched_files = {}
    logger.debug("Looking for exact matches")
    for filename, size in list(path1_files.items()):
        if filename in path2_files.keys() and path2_files[filename] == size:
            matched_files[filename] = filename
            del path1_files[filename]
            del path2_files[filename]
    logger.debug(
        "{0} of {1} files matched".format(
            len(matched_files), len(path2_files) + len(matched_files)
        )
    )

    logger.debug(
        "Looking for matches with same size and name but different root folder"
    )
    for filename1, size1 in list(path1_files.items()):
        no_root1 = os.sep.join(os.path.normpath(filename1).split(os.sep)[1:])
        for filename2, size2 in list(list(path2_files.items())):
            no_root2 = os.sep.join(os.path.normpath(filename2).split(os.sep)[1:])
            if no_root1 == no_root2 and size1 == size2:
                matched_files[filename1] = filename2
                del path1_files[filename1]
                del path2_files[filename2]
                break
    logger.debug(
        "{0} of {1} files matched".format(
            len(matched_files), len(path2_files) + len(matched_files)
        )
    )

    logger.debug("Looking for matches with same base name and size")
    for filename1, size1 in list(path1_files.items()):
        for filename2, size2 in list(path2_files.items()):
            if (
                os.path.basename(filename1) == os.path.basename(filename2)
                and size1 == size2
            ):
                if os.path.basename(filename1) not in [
                    os.path.basename(p) for p in path2_files.keys()
                ]:
                    matched_files[filename1] = filename2
                    del path1_files[filename1]
                    del path2_files[filename2]
                    break
    logger.debug(
        "{0} of {1} files matched".format(
            len(matched_files), len(path2_files) + len(matched_files)
        )
    )

    logger.debug("Looking for matches by size only")
    for filename1, size1 in list(path1_files.items()):
        for filename2, size2 in list(path2_files.items()):
            logger.debug(
                "Comparing size of {0} ({1}) to size of {2} ({3})".format(
                    filename1, size1, filename2, size2
                )
            )
            if size1 == size2:
                logger.debug("Matched {0} to {1}".format(filename1, filename2))
                matched_files[filename1] = filename2
                del path1_files[filename1]
                del path2_files[filename2]
                break
    logger.debug(
        "{0} of {1} files matched".format(
            len(matched_files), len(path2_files) + len(matched_files)
        )
    )

    if len(path2_files) > 0:
        logger.info("Not all files could be matched, returning...")
        return Match(None)
    return Match(torrent.ID, Path(os.path.dirname(path1)), matched_files)


def match_by_movie(movie, filepath) -> Match:
    """Tries to match a torrent against a single movie"""
    logger = logging.getLogger(__name__)
    logger.info(
        "Attempting to match against movie {0} ({1})".format(movie.ID, movie["Title"])
    )

    movie.load_html_data()
    for torrent in movie["Torrents"]:
        match = match_by_torrent(torrent, os.path.abspath(filepath))
        if match:
            return match
    return Match(None)


def match_by_guessed_name(ptp, filepath, limit, name=None) -> Match:
    """Use guessit to find the movie by metadata scraped from the filename"""
    logger = logging.getLogger(__name__)
    filepath = os.path.abspath(filepath)
    try:
        import guessit  # pylint: disable=import-error
    except ImportError:
        logger.warning("Error importing 'guessit' module, skipping name guess")
        return Match(None)
    logger.info("Guessing name from filepath with guessit")
    if not name:
        name = os.path.basename(filepath)
    guess = guessit.guessit(name)
    if "title" in guess and guess["title"]:
        search_params = {"searchstr": guess["title"]}
        if "year" in guess:
            search_params["year"] = guess["year"]
        movies = ptp.search(search_params)
        if len(movies) == 0:
            movies = ptp.search({"searchstr": guess["title"], "inallakas": "1"})
        if len(movies) == 0:
            logger.debug("Could not find any movies by search with a guessed name")
        for movie in movies[:limit]:
            match = match_by_movie(movie, filepath)
            if match:
                return match
    return Match(None)


def match_against_file(ptp, filepath, movie_limit) -> Match:
    """Use's PTP's file search feature to match a filename to a movie"""
    logger = logging.getLogger(__name__)
    filepath = os.path.abspath(filepath)
    logger.info("Searching movies by file list")
    for movie in ptp.search({"filelist": os.path.basename(filepath)})[:movie_limit]:
        match = match_by_movie(movie, filepath)
        if match:
            return match
    return Match(None)


def create_matched_files(match, directory=None, action="hard", dry_run=False):
    """Intelligently create any necessary files or directories by different methods"""
    logger = logging.getLogger(__name__)
    if dry_run:
        logger.info("Dry run, no files or directories will be created")
    for origin_file, matched_file in match.matched_files.items():
        origin_file = os.path.join(match.path, origin_file)
        if directory is None:
            directory = match.path
        file_to_create = os.path.join(directory, matched_file)
        path_to_create = os.path.dirname(file_to_create)
        try:
            logger.debug("Creating directory '{0}'".format(path_to_create))
            if not dry_run:
                os.makedirs(path_to_create)
        except OSError as exc:
            # Ignore OSError only if the directory already exists
            if exc.errno != 17:
                raise
        if os.path.lexists(file_to_create):
            logger.debug(
                "File '{0}' already exists, skipping creation".format(file_to_create)
            )
            continue
        logger.info(
            "Creating file '{0}' from '{1}' via action '{2}'".format(
                file_to_create, origin_file, action
            )
        )
        if not dry_run:
            if action == "soft":
                os.symlink(origin_file, file_to_create)
            elif action == "hard":
                os.link(origin_file, file_to_create)
            elif action == "skip":
                continue
    match.path = directory
    return match


def load_torrent(
    ID, path, client=None, hash_check=False, overwrite_incomplete=False
) -> bool:
    """Send a torrent to rtorrent and kick off the hash recheck"""
    logger = logging.getLogger(__name__)
    torrent = ptpapi.Torrent(ID=ID)
    torrent_data = torrent.download()
    data = metafile.Metafile(bencode.bdecode(torrent_data))
    thash = data.info_hash()
    path = Path(path)
    if hash_check:
        from pyrosimple.util.metafile import PieceFailer

        logger.debug("Starting hash check against %r", str(path))
        pf = PieceFailer(data)
        try:
            data.hash_check(path, piece_callback=pf.check_piece)
            data.add_fast_resume(path)
        except OSError as exc:
            logger.error("Could not complete hash check: %s", exc)
            return False
    if client is None:
        hash_exists = True
        proxy = pyrosimple.connect().open()
        try:
            logger.debug("Testing for hash {0}".format(proxy.d.hash(thash)))
            if proxy.d.complete(thash):
                logger.error(
                    "Hash {0} is already completed in rtorrent as {1}, cannot load.".format(
                        thash, proxy.d.name(thash)
                    )
                )
                return False
            hash_exists = True
        except rpc.HashNotFound:
            pass
        if hash_exists and not overwrite_incomplete:
            logger.error(
                "Hash {0} already exists in rtorrent as {1}, cannot load.".format(
                    thash, proxy.d.name(thash)
                )
            )
            return False
        proxy.load.raw("", xmlrpc_client.Binary(torrent_data))
        # Wait until the torrent is loaded and available
        while True:
            sleep(1)
            try:
                proxy.d.hash(thash)
                break
            except (xmlrpc_client.Fault, rpc.HashNotFound):
                pass
        logger.info("Torrent loaded at %r", str(path))
        proxy.d.custom.set(thash, "tm_completed", str(int(time())))
        proxy.d.directory.set(thash, str(path))
        proxy.d.check_hash(thash)
        return True
    elif isinstance(client, str) and client.startswith("file://"):
        dest = Path(client[7:], data["info"]["name"] + ".torrent").expanduser()
        logger.info("Saving file to %r", str(dest))
        data.save(dest)
    else:
        bd = bencodepy.BencodeDecoder()
        return bool(client.add(bd.decode(torrent_data), path))


def define_parser():
    """Define the arguments for the CLI"""
    parser = argparse.ArgumentParser(
        description="Attempt to find and reseed torrents on PTP"
    )
    parser.add_argument("-u", "--url", help="Permalink to the torrent page")
    parser.add_argument(
        "files",
        help="Paths to files/directories to reseed (or leave blank to read stdin)",
        nargs="*",
        type=str,
    )
    parser.add_argument(
        "-n",
        "--dry-run",
        help="Don't actually create any files or load torrents",
        action="store_true",
    )
    parser.add_argument(
        "-a",
        "--action",
        help="Method to use when creating files",
        choices=["hard", "soft", "skip"],
        default=ptpapi.config.config.get("Reseed", "action"),
    )
    parser.add_argument(
        "-d",
        "--create-in-directory",
        help="Directory to create any new files in, if necessary",
        default=None,
    )
    parser.add_argument(
        "--debug",
        help="Print lots of debugging statements",
        action="store_const",
        dest="loglevel",
        const=logging.DEBUG,
        default=logging.WARNING,
    )
    parser.add_argument(
        "--client",
        help="Experimental: use a custom libtc URL. See https://github.com/JohnDoee/libtc#url-syntax for examples",
        nargs=1,
    )
    parser.add_argument(
        "-v",
        "--verbose",
        help="Be verbose",
        action="store_const",
        dest="loglevel",
        const=logging.INFO,
    )
    parser.add_argument(
        "-q",
        "--quiet",
        help="Don't show any messages",
        action="store_const",
        dest="loglevel",
        const=logging.CRITICAL,
    )
    parser.add_argument(
        "-s", "--summary", help="Show a summary of all actions", action="store_true"
    )
    parser.add_argument(
        "-l",
        "--limit",
        help="Limit the maximum number of movies checked for each file",
        type=int,
        default=5,
    )
    parser.add_argument(
        "--hash-check",
        help="Hash check against any found matches before loading",
        action="store_true",
    )
    parser.add_argument(
        "--overwrite-incomplete",
        help="If the torrent exists as incomplete, change the path of the existing torrent (rtorrent only)",
        action="store_true",
    )
    return parser


def process(cli_args):
    """The entrypoint"""
    parser = define_parser()
    args = parser.parse_args(cli_args)
    logger = logging.getLogger("ptp-reseed")

    logging.basicConfig(level=args.loglevel)

    # Futile attempt to impose our loglevel upon pyroscope
    logging.basicConfig(level=args.loglevel)

    # Load PTP API
    ptp = ptpapi.login()

    loaded = []
    would_load = []
    could_not_load = []
    not_found = []

    if args.files in (["-"], []):
        filelist = sys.stdin
    else:
        filelist = args.files

    loaded_paths = []

    if args.client:
        if args.client[0].startswith("file://"):
            client = args.client[0]
        else:
            client = libtc.parse_libtc_url(args.client[0])
    else:
        client = None

    for filename in filelist:
        match = Match(None)
        filename = filename.strip("\n")

        logger.info('Starting reseed attempt on file "{0}"'.format(filename))

        if not os.path.exists(filename):
            logger.error("File/directory {0} does not exist".format(filename))
            continue

        if args.url:
            parsed_url = parse_qs(urlparse(args.url).query)
            if "torrentid" in parsed_url:
                match = match_by_torrent(
                    ptpapi.Torrent(ID=parsed_url["torrentid"][0]), filename.encode()
                )
            elif "id" in parsed_url:
                match = match_by_movie(
                    ptpapi.Movie(ID=parsed_url["id"][0]), filename.encode()
                )
        elif filename:
            for match_type in ptpapi.config.config.get("Reseed", "findBy").split(","):
                try:
                    if match_type == "filename":
                        if os.path.abspath(filename) in loaded_paths:
                            logger.info(
                                "Path {0} already in rtorrent, skipping".format(
                                    os.path.abspath(filename)
                                )
                            )
                        else:
                            logger.debug(
                                "Path {0} not in rtorrent".format(
                                    os.path.abspath(filename)
                                )
                            )
                            match = match_against_file(ptp, filename, args.limit)
                    elif match_type == "title":
                        match = match_by_guessed_name(ptp, filename, args.limit)
                    else:
                        logger.error(
                            "Match type {0} not recognized for {1}, skipping".format(
                                match_type, filename
                            )
                        )
                    if match:
                        break
                except Exception:
                    print("Error while attempting to match file '{0}'".format(filename))
                    raise

        # Make sure we have the minimum information required
        if not match:
            not_found.append(filename)
            logger.error(
                "Could not find an associated torrent for '{0}', cannot reseed".format(
                    filename
                )
            )
            continue

        if args.create_in_directory:
            create_in = args.create_in_directory
        elif ptpapi.config.config.has_option("Reseed", "createInDirectory"):
            create_in = ptpapi.config.config.get("Reseed", "createInDirectory")
        else:
            create_in = None
        create_matched_files(
            match, directory=create_in, action=args.action, dry_run=args.dry_run
        )
        logger.info(
            "Found match, now loading torrent {0} to path {1}".format(
                match.ID, match.path
            )
        )
        match_log_line = (
            f"https://passthepopcorn.me/torrents.php?torrentid={match.ID} -> {filename}"
        )
        if args.dry_run:
            would_load.append(match_log_line)
            logger.debug("Dry-run: Stopping before actual load")
            continue
        if load_torrent(
            match.ID,
            Path(match.path),
            client,
            hash_check=args.hash_check,
            overwrite_incomplete=args.overwrite_incomplete,
        ):
            loaded.append(match_log_line)
        else:
            could_not_load.append(match_log_line)

    if args.summary:
        if loaded:
            print("==> Loaded:")
            print("\n".join(loaded))
        if would_load:
            print("==> Would have loaded:")
            print("\n".join(would_load))
        if could_not_load:
            print("==> Could not load:")
            print("\n".join(could_not_load))
        if not_found:
            print("==> Not found:")
            print("\n".join(not_found))

    exit_code = 0
    if len(not_found) == 1:
        exit_code = 1
    elif len(not_found) > 1:
        exit_code = 2
    elif len(could_not_load) > 0:
        exit_code = 3

    logger.debug(
        "Total session tokens consumed: %s", ptpapi.session.session.consumed_tokens
    )
    logger.debug("Exiting...")
    return exit_code


def main():
    # Load pyroscope
    exit_code = process(sys.argv[1:])
    sys.exit(exit_code)


if __name__ == "__main__":
    main()



"""Represents a movie"""
import logging
import operator
import os.path
import re

from datetime import datetime

from bs4 import BeautifulSoup as bs4  # pylint: disable=import-error

from ptpapi import torrent
from ptpapi.error import PTPAPIException
from ptpapi.session import session
from ptpapi.util import human_to_bytes


LOGGER = logging.getLogger(__name__)


class Movie:
    """A class representing a movie"""

    def __init__(self, ID=None, data=None):
        self.torrents = []
        self.key_finder = {
            "json": [
                "ImdbId",
                "ImdbRating",
                "ImdbVoteCount",
                "Torrents",
                "CoverImage",
                "Name",
            ],
            "html": [
                "Title",
                "Year",
                "Cover",
                "Tags",
                "Directors",
                "PtpRating",
                "PtpVoteCount",
                "UserRating",
                "Seen",
                "Snatched",
            ],
            "inferred": ["Link", "Id", "GroupId"],
        }

        if data:
            self.data = data
            self.conv_json_torrents()
            self.ID = data["GroupId"]  # pylint: disable=invalid-name
        elif ID:
            self.ID = ID
            self.data = {}
        else:
            raise PTPAPIException("Could not load necessary data for Movie class")

    def __repr__(self):
        return "<ptpapi.Movie ID %s>" % self.ID

    def __str__(self):
        return "<ptpapi.Movie ID %s>" % self.ID

    def __getitem__(self, name):
        if name not in self.data or self.data[name] is None:
            for key, val in self.key_finder.items():
                if name in val:
                    getattr(self, "load_%s_data" % key)()
        return self.data[name]

    def items(self):
        """Passthru function for underlying dict"""
        return self.data.items()

    def update(self, obj):
        for k, v in obj.items():
            self.data[k] = v

    def __setitem__(self, key, value):
        self.data[key] = value

    def load_inferred_data(self):
        self.data["Id"] = self.ID
        self.data["GroupId"] = self.ID
        self.data["Link"] = "https://passthepopcorn.me/torrents.php?id=" + self.ID

    def load_json_data(self):
        """Load movie JSON data"""
        self.data.update(
            session.base_get("torrents.php", params={"id": self.ID, "json": "1"}).json()
        )
        if "ImdbId" not in self.data:
            self.data["ImdbId"] = ""
        self.conv_json_torrents()

    def conv_json_torrents(self):
        """Util function to normalize data"""
        if self.data["Torrents"]:
            torrents = self.data["Torrents"]
            for t in torrents:
                if "RemasterTitle" not in t:
                    t["RemasterTitle"] = ""
            self.data["Torrents"] = [torrent.Torrent(data=t) for t in torrents]

    def load_html_data(self):
        """Scrape all data from a movie's HTML page"""
        soup = bs4(
            session.base_get("torrents.php", params={"id": self.ID, "json": 0}).text,
            "html.parser",
        )
        self.data["Cover"] = soup.find("img", class_="sidebar-cover-image")["src"]
        # Title and Year
        match = re.match(
            rb"(.*)(:? \[(\d{4})\])?",
            soup.find("h2", class_="page__title").encode_contents(),
        )
        self.data["Title"] = match.group(1)
        self.data["Year"] = "0" if match.group(2) is None else match.group(2)
        # Genre tags
        self.data["Tags"] = []
        for tagbox in soup.find_all("div", class_="box_tags"):
            for tag in tagbox.find_all("li"):
                self.data["Tags"].append(str(tag.find("a").string))
        # Directors
        self.data["Directors"] = []
        for director in soup.find("h2", class_="page__title").find_all(
            "a", class_="artist-info-link"
        ):
            self.data["Directors"].append({"Name": director.string.strip()})
        # Ratings
        rating = soup.find(id="ptp_rating_td")
        self.data["PtpRating"] = rating.find(id="user_rating").text.strip("%")
        self.data["PtpRatingCount"] = re.sub(
            r"\D", "", rating.find(id="user_total").text
        )
        your_rating = rating.find(id="ptp_your_rating").text
        if "?" in your_rating:
            self.data["UserRating"] = None
            self.data["Seen"] = False
        elif re.sub(r"\D", "", your_rating) == "":
            self.data["UserRating"] = None
            self.data["Seen"] = True
        else:
            self.data["UserRating"] = re.sub(r"\D", "", your_rating)
            self.data["Seen"] = True
        # Have we snatched this
        self.data["Snatched"] = False
        if soup.find(class_="torrent-info-link--user-snatched") or soup.find(
            class_="torrent-info-link--user-seeding"
        ):
            self.data["Snatched"] = True

        # File list & trumpability for torrents
        for tor in self["Torrents"]:
            # Get file list
            filediv = soup.find("div", id="files_%s" % tor.ID)
            tor.data["Filelist"] = {}
            basepath = re.match(
                r"\/(.*)\/", filediv.find("thead").find_all("div")[1].get_text()
            ).group(1)
            for elem in filediv.find("tbody").find_all("tr"):
                try:
                    bytesize = (
                        elem("td")[1]("span")[0]["title"]
                        .replace(",", "")
                        .replace(" bytes", "")
                    )
                except IndexError:
                    LOGGER.error(
                        "Could not parse site for filesize, possibly check for bad filenames: https://passthepopcorn.me/torrents.php?torrentid=%s",
                        tor.ID,
                    )
                    continue
                filepath = os.path.join(basepath, elem("td")[0].string)
                tor.data["Filelist"][filepath] = bytesize
            # Check if trumpable
            if soup.find(id="trumpable_%s" % tor.ID):
                tor.data["Trumpable"] = [
                    s.get_text()
                    for s in soup.find(id="trumpable_%s" % tor.ID).find_all("span")
                ]
            else:
                tor.data["Trumpable"] = []

    def best_match(self, profile):
        """A function to pull the best match of a movie, based on a human-readable filter

        :param profile: a filter string
        :rtype: The best matching movie, or None"""
        # We're going to emulate what.cd's collector option
        profiles = profile.lower().split(",")
        current_sort = None
        if "Torrents" not in self.data:
            self.load_json_data()
        for subprofile in profiles:
            LOGGER.debug("Attempting to match movie to profile '%s'", subprofile)
            matches = self.data["Torrents"]
            simple_filter_dict = {
                "gp": (lambda t, _: t["GoldenPopcorn"]),
                "scene": (lambda t, _: t["Scene"]),
                "576p": (lambda t, _: t["Resolution"] == "576p"),
                "480p": (lambda t, _: t["Resolution"] == "480p"),
                "720p": (lambda t, _: t["Resolution"] == "720p"),
                "1080p": (lambda t, _: t["Resolution"] == "1080p"),
                "HD": (lambda t, _: t["Quality"] == "High Definition"),
                "SD": (lambda t, _: t["Quality"] == "Standard Definition"),
                "not-remux": (lambda t, _: "remux" not in t["RemasterTitle"].lower()),
                "remux": (lambda t, _: "remux" in t["RemasterTitle"].lower()),
                "x264": (lambda t, _: t["Codec"] == "x264"),
                "xvid": (lambda t, _: t["Codec"] == "XviD"),
                "seeded": (lambda t, _: int(t["Seeders"]) > 0),
                "not-trumpable": (lambda t, _: not t["Trumpable"]),
                "unseen": (lambda t, m: not m["Seen"]),
                "unsnatched": (lambda t, m: not m["Snatched"]),
            }
            for (name, func) in simple_filter_dict.items():
                if name.lower() in subprofile.split(" "):
                    matches = [t for t in matches if func(t, self)]
                    LOGGER.debug(
                        "%i matches after filtering by parameter '%s'",
                        len(matches),
                        name,
                    )
            # lambdas that take a torrent, a function for comparison, and a value-as-a-string
            comparative_filter_dict = {
                "seeders": (lambda t, f, v: f(int(t["Seeders"]), int(v))),
                "size": (
                    lambda t, f, v: f(
                        int(t["Size"]), human_to_bytes(v, case_sensitive=False)
                    )
                ),
            }
            comparisons = {
                ">": operator.gt,
                ">=": operator.ge,
                "=": operator.eq,
                "==": operator.eq,
                "!=": operator.ne,
                "<>": operator.ne,
                "<": operator.lt,
                "<=": operator.le,
            }
            for (name, func) in comparative_filter_dict.items():
                match = re.search(r"\b%s([<>=!]+)(.+?)\b" % name, subprofile)
                if match is not None:
                    comp_func = comparisons[match.group(1)]
                    value = match.group(2)
                    matches = [t for t in matches if func(t, comp_func, value)]
                    LOGGER.debug(
                        "%i matches after filtering by parameter '%s'",
                        len(matches),
                        name,
                    )
            sort_dict = {
                "most recent": (
                    True,
                    (lambda t: datetime.strptime(t["UploadTime"], "%Y-%m-%d %H:%M:%S")),
                ),
                "smallest": (
                    False,
                    (lambda t: human_to_bytes(t["Size"], case_sensitive=False)),
                ),
                "most seeders": (True, (lambda t: int(t["Seeders"]))),
                "largest": (
                    True,
                    (lambda t: human_to_bytes(t["Size"], case_sensitive=False)),
                ),
            }
            if len(matches) == 1:
                return matches[0]
            elif len(matches) > 1:
                for name, (rev, sort) in sort_dict.items():
                    if name in subprofile:
                        current_sort = name
                if current_sort is None:
                    current_sort = "most recent"
                LOGGER.debug("Sorting by parameter %s", current_sort)
                (rev, sort) = sort_dict[current_sort]
                return sorted(matches, key=sort, reverse=rev)[0]
        LOGGER.info("Could not find best match for movie %s", self.ID)
        return None



#!/usr/bin/env python
import argparse
import io
import logging
import re
import sys
import textwrap

from pathlib import Path
from urllib.parse import urlparse

import requests
import ruamel.yaml
import urllib3

from bs4 import BeautifulSoup as bs4
from pyrosimple.util import metafile

import ptpapi


YAML = ruamel.yaml.YAML()
YAML.top_level_colon_align = True
YAML.width = float("inf")
YAML.allow_unicode = True
YAML.encoding = "utf-8"

RE_COMMENT = re.compile(
    r"https://passthepopcorn.me/torrents.php\?id=(\d+)&torrentid=(\d+)"
)
RE_URL = re.compile(
    r"((http|https)\:\/\/)[a-zA-Z0-9\.\/\?\:@\-_=#]+\.([a-zA-Z]){2,6}([a-zA-Z0-9\.\&\/\?\:@\-_=#])*"
)
RE_DELETED_BY = re.compile(r"was deleted by .* for")


def write_origin(t, args):
    logger = logging.getLogger(__name__)
    mfile_path = Path(t)
    mfile = metafile.Metafile.from_file(mfile_path)
    if "comment" not in mfile or not RE_COMMENT.match(mfile["comment"]):
        logger.info("Skipping file %s, does not contain PTP URL in comment", t)
        return
    logger.info("Working file %s", t)
    match = RE_COMMENT.match(mfile["comment"])
    movie = ptpapi.Movie(match.group(1))
    torrent = ptpapi.Torrent(data={"Id": match.group(2), "GroupId": match.group(1)})
    if args.output_directory is not None:
        output_dir = args.output_directory
    else:
        output_dir = Path(mfile_path.stem)
    output_dir.mkdir(parents=True, exist_ok=True)
    yaml_path = Path(output_dir, mfile_path.with_suffix(".yaml").name)
    if yaml_path.exists() and not args.overwrite:
        logger.info("Skipping file %s, origin file '%s' exists", t, yaml_path)
        return
    nfo_path = Path(output_dir, mfile_path.with_suffix(".nfo").name)
    logger.info("Writing origin YAML file %s", yaml_path)
    output = "---\n"
    # Basic data
    data = {
        "Title": movie["Name"],
        "Year": int(movie["Year"]),
        "Directors": movie["Directors"],
        "ReleaseName": torrent["ReleaseName"],
        "RemasterTitle": torrent["RemasterTitle"],
        "IMDb": f'https://imdb.com/title/tt{movie["ImdbId"]}',
        "Cover": movie["Cover"],
        "Permalink": mfile["comment"],
        "InfoHash": torrent["InfoHash"],
        "Codec": torrent["Codec"],
        "Container": torrent["Container"],
        "UploadTime": torrent["UploadTime"],
        "Checked": torrent["Checked"],
        "GoldenPopcorn": torrent["GoldenPopcorn"],
        "Scene": torrent["Scene"],
        "ReleaseGroup": torrent["ReleaseGroup"],
        "Resolution": torrent["Resolution"],
        "Size": int(torrent["Size"]),
        "Source": torrent["Source"],
        "Tags": movie["Tags"],
    }
    buf = io.StringIO()
    YAML.dump(data, buf)
    output += buf.read()
    # Nicely format multi-line descriptions
    desc = torrent["BBCodeDescription"]
    output += "Description: |\n"
    output += textwrap.indent(desc, "")
    output += "\n"
    # Scrubbed deletion log
    soup = bs4(
        ptpapi.session.session.base_get(
            "torrents.php",
            params={
                "action": "history_log",
                "groupid": movie.ID,
                "search": "",
                "only_deletions": 1,
            },
        ).content,
        features="html.parser",
    )
    log_body = soup.find("tbody")
    log_data = []
    for row in log_body.find_all("tr"):
        time = row.find_all("span")[0]["title"]
        message = RE_DELETED_BY.sub("was deleted for", row.find_all("span")[1].text)
        if message != row.find_all("span")[1].text:
            log_data.append({"Time": time, "Message": message.strip()})
    buf = io.StringIO()
    YAML.dump({"Log": log_data}, buf)
    output += buf.read()
    with yaml_path.open("w", encoding="utf-8") as stream:
        stream.write(output)
    # NFO
    if "Nfo" in torrent.data and torrent["Nfo"]:
        logger.info("Writing NFO file %s", nfo_path)
        nfo_path.write_text(torrent["Nfo"], encoding="utf-8")
    # Download anything that looks like a URL
    if not args.no_images:
        for m in re.finditer(RE_URL, desc):
            url_parts = urlparse(m.group(0))
            path = Path(output_dir, Path(url_parts.path).name)
            # Skip IMDb title URLS
            url = m.group(0)
            if "imdb.com/title/" not in url and "passthepopcorn.me" not in url:
                if not path.exists() or args.overwrite:
                    logger.info("Downloading description image %s to %s", url, path)
                    try:
                        resp = requests.get(url, timeout=30)
                    except (
                        requests.exceptions.RequestException,
                        urllib3.exceptions.HTTPError,
                    ) as exc:
                        logger.error("Could not fetch URL %s: %s", url, exc)
                        continue
                    if "Content-Type" in resp.headers and resp.headers[
                        "Content-Type"
                    ].startswith("image"):
                        with path.open("wb") as fh:
                            fh.write(resp.content)
        # Cover
        url_parts = urlparse(movie["Cover"])
        path = Path(output_dir, Path(url_parts.path).name)
        if not path.exists():
            logger.info("Downloading cover %s to %s", movie["Cover"], path)
            try:
                resp = requests.get(movie["Cover"], timeout=30)
            except (
                requests.exceptions.RequestException,
                urllib3.exceptions.HTTPError,
            ) as exc:
                logger.error("Could not fetch cover URL %s: %s", movie["Cover"], exc)
                return
            if "Content-Type" not in resp.headers:
                logger.warning("Cover did not return an content-type, cannot save")
            if resp.headers["Content-Type"].startswith("image"):
                with path.open("wb") as fh:
                    fh.write(resp.content)
            else:
                logger.warning(
                    "Cover did not appear to be an image (content-type %s), not saving",
                    resp.headers["Content-Type"],
                )


def main():
    parser = argparse.ArgumentParser(
        usage="Download metadata from PTP for archival purposes"
    )
    parser.add_argument(
        "torrent", nargs="+", help="Torrent file to use for scraping information"
    )
    parser.add_argument(
        "--debug",
        help="Print lots of debugging statements",
        action="store_const",
        dest="loglevel",
        const=logging.DEBUG,
        default=logging.WARNING,
    )
    parser.add_argument(
        "-v",
        "--verbose",
        help="Be verbose",
        action="store_const",
        dest="loglevel",
        const=logging.INFO,
    )
    parser.add_argument(
        "-q",
        "--quiet",
        help="Hide most messages",
        action="store_const",
        dest="loglevel",
        const=logging.CRITICAL,
    )
    parser.add_argument(
        "-r", "--recursive", help="Recursively walk directory", action="store_true"
    )
    parser.add_argument(
        "--overwrite",
        help="Re-download files even if they already exist",
        action="store_true",
    )
    parser.add_argument(
        "-d",
        "--output-directory",
        help="Directory to write files to (defaults to torrent name without extension)",
        metavar="DIR",
    )
    parser.add_argument(
        "--no-images", help="Skip downloading images", action="store_true"
    )
    args = parser.parse_args()
    logging.basicConfig(level=args.loglevel)
    logger = logging.getLogger(__name__)
    ptpapi.login()
    for p in args.torrent:
        p_path = Path(p)
        if p_path.is_dir():
            if args.recursive:
                for t in p_path.rglob("*.torrent"):
                    try:
                        write_origin(t, args)
                    except Exception:
                        logger.error("Error handling file %s", t)
                        raise
            else:
                logger.warning(
                    "Skipping directory %s, use --recursive to descend into directories",
                    p,
                )
        if p_path.is_file():
            try:
                write_origin(p, args)
            except Exception:
                logger.error("Error handling file %s", p)
                raise


if __name__ == "__main__":
    main()



#!/bin/env python
"""The entrypoint module for access the API"""
import html
import logging
import os
import pickle
import re

from pathlib import Path

import requests

from bs4 import BeautifulSoup as bs4

from ptpapi import util
from ptpapi.config import config
from ptpapi.error import PTPAPIException
from ptpapi.movie import Movie
from ptpapi.session import session
from ptpapi.user import CurrentUser


LOGGER = logging.getLogger(__name__)


def login(kwargs):
    """Simple helper function"""
    return API(**kwargs)


class API:
    """Used for instantiating an object that can access the API"""

    def __init__(
        self, username=None, password=None, passkey=None, api_user=None, api_key=None
    ):
        self.current_user_id = None
        j = None
        self.cookies_file = Path(config.get("Main", "cookiesFile"))
        logger = logging.getLogger(__name__)

        if config.has_option("PTP", "ApiUser") and config.has_option("PTP", "ApiKey"):
            pass
        elif (
            config.has_option("PTP", "password")
            and config.has_option("PTP", "username")
            and config.has_option("PTP", "passkey")
        ):
            logger.warning(
                "Using your password/passkey to access the site is deprecated, "
                + "see README.md for instructions on using the new ApiUser/ApiKey."
            )
        else:
            raise PTPAPIException("No credentials found!")

        req = None

        if config.has_option("PTP", "ApiUser") and api_user is None and api_key is None:
            session.headers.update(
                {
                    "ApiUser": config.get("PTP", "ApiUser"),
                    "ApiKey": config.get("PTP", "ApiKey"),
                }
            )
        elif api_user is not None and api_key is not None:
            session.headers.update(
                {
                    "ApiUser": api_user,
                    "ApiKey": api_key,
                }
            )
        elif self.cookies_file.is_file():
            LOGGER.debug("Initiating login sequence.")
            self.__load_cookies()
            # A really crude test to see if we're logged in
            session.max_redirects = 1
            try:
                req = session.base_get("torrents.php")
                util.raise_for_cloudflare(req.text)
            except requests.exceptions.TooManyRedirects:
                if self.cookies_file.is_file():
                    self.cookies_file.unlink()
                session.cookies = requests.cookies.RequestsCookieJar()
            session.max_redirects = 3
        # If we're not using the new method and we don't have a cookie, get one
        if not config.has_option("PTP", "ApiUser") and not self.cookies_file.is_file():
            password = password or config.get("PTP", "password")
            username = username or config.get("PTP", "username")
            passkey = passkey or config.get("PTP", "passkey")
            if not password or not passkey or not username:
                raise PTPAPIException("Not enough info provided to log in.")
            try:
                req = session.base_post(
                    "ajax.php?action=login",
                    data={
                        "username": username,
                        "password": password,
                        "passkey": passkey,
                    },
                )
                j = req.json()
            except ValueError:
                if req.status_code == 200:
                    raise PTPAPIException(
                        "Could not parse returned json data."
                    ) from ValueError
                if req.status_code == 429:
                    LOGGER.critical(req.text.strip())
                req.raise_for_status()
            if j["Result"] == "TfaRequired":
                req = session.base_post(
                    "ajax.php?action=login",
                    data={
                        "username": username,
                        "password": password,
                        "passkey": passkey,
                        "TfaType": "normal",
                        "TfaCode": input("Enter 2FA auth code:"),
                    },
                )
                j = req.json()
            if j["Result"] != "Ok":
                raise PTPAPIException(
                    "Failed to log in. Please check the username, password and passkey. Response: %s"
                    % j
                )
            self.__save_cookie()
            # Get some information that will be useful for later
            req = session.base_get("index.php")
            util.raise_for_cloudflare(req.text)
            LOGGER.info("Login successful.")

    def is_api(self):
        """Helper function to check for the use of ApiUser"""
        return config.has_option("PTP", "ApiKey")

    def logout(self):
        """Forces a logout. In ApiUser mode, essentially a waste of two request tokens."""
        req = session.base_get("index.php")
        auth_key = re.search(r"auth=([0-9a-f]{32})", req.text).group(1)
        self.cookies_file.unlink()
        return session.base_get("logout.php", params={"auth": auth_key})

    def __save_cookie(self):
        """Save requests' cookies to a file"""
        with self.cookies_file.open("wb") as fileh:
            LOGGER.debug("Pickling HTTP cookies to %s", self.cookies_file)
            pickle.dump(requests.utils.dict_from_cookiejar(session.cookies), fileh)

    def __load_cookies(self):
        """Reload requests' cookies"""
        with self.cookies_file.open("rb") as fileh:
            LOGGER.debug("Unpickling HTTP cookies from file %s", self.cookies_file)
            session.cookies = requests.utils.cookiejar_from_dict(pickle.load(fileh))

    def current_user(self):
        """Function to get the current user"""
        # TODO: See if it can be scraped earlier without an extra request
        if self.current_user_id is None:
            req = session.base_get("index.php")
            self.current_user_id = re.search(r"user.php\?id=(\d+)", req.text).group(1)
        return CurrentUser(self.current_user_id)

    def search(self, filters, add_coverview_data=False):
        """Perform a movie search"""
        if "name" in filters:
            filters["searchstr"] = filters["name"]
        filters["json"] = "noredirect"
        ret_array = []
        for movie in session.base_get("torrents.php", params=filters).json()["Movies"]:
            if "Directors" not in movie:
                movie["Directors"] = []
            if "ImdbId" not in movie:
                movie["ImdbId"] = "0"
            movie["Title"] = html.unescape(movie["Title"])
            ret_array.append(Movie(data=movie))
        return ret_array

    # There's probably a better place to put this, but it's not really useful inside the Movie class
    search_coverview_fields = [
        "RtRating",
        "RtUrl",
        "UserRating",
        "TotalSeeders",
        "TotalSnatched",
        "TotalLeechers",
    ]

    def search_coverview(self, filters):
        filters["json"] = 0
        ret_array = []
        if "name" in filters:
            filters["searchstr"] = filters["name"]
        for movie in util.snarf_cover_view_data(
            session.base_get("torrents.php", params=filters).content, key=b"PageData"
        ):
            if "UserRating" not in movie:
                movie["UserRating"] = None
            ret_array.append(Movie(data=movie))
        return ret_array

    def search_single(self, filters):
        """If you know ahead of time that a filter will redirect to a single movie,
        you can use this method to avoid an exception until that behavior is
        fixed upstream."""
        if "name" in filters:
            filters["searchstr"] = filters["name"]
        filters["json"] = "noredirect"
        resp = session.base_get("torrents.php", params=filters)
        movie_id = re.search(r"id=([0-9]+)", resp.url)
        if movie_id is not None:
            return Movie(ID=movie_id.group(1))
        else:
            return None

    def upload_info(self):
        """Scrape as much info as possible from upload.php"""
        data = {}
        soup = bs4(session.base_get("upload.php").content, "html.parser")
        data["announce"] = soup.find_all(
            "input",
            type="text",
            value=re.compile("http://please.passthepopcorn.me:2710/.*/announce"),
        )[0]["value"]
        data["subtitles"] = {}
        label_list = [
            u.find_all("label") for u in soup.find_all(class_="languageselector")
        ]
        labels = [item for sublist in label_list for item in sublist]
        for l in labels:
            data["subtitles"][l["for"].lstrip("subtitle_")] = l.get_text().strip()
        data["remaster_title"] = [
            a.get_text() for a in soup.find(id="remaster_tags").find_all("a")
        ]
        data["resolutions"] = [
            o.get_text() for o in soup.find(id="resolution").find_all("option")
        ]
        data["containers"] = [
            o.get_text() for o in soup.find(id="container").find_all("option")
        ]
        data["sources"] = [
            o.get_text() for o in soup.find(id="source").find_all("option")
        ]
        data["codecs"] = [
            o.get_text() for o in soup.find(id="codec").find_all("option")
        ]
        data["tags"] = [
            o.get_text() for o in soup.find(id="genre_tags").find_all("option")
        ]
        data["categories"] = [
            o.get_text() for o in soup.find(id="categories").find_all("option")
        ]
        data["AntiCsrfToken"] = soup.find("body")["data-anticsrftoken"]
        return data

    def need_for_seed(self, filters=None):
        """List torrents that need seeding"""
        if filters is None:
            filters = {}
        data = util.snarf_cover_view_data(
            session.base_get("needforseed.php", params=filters).content
        )
        torrents = []
        for m in data:
            torrent = m["GroupingQualities"][0]["Torrents"][0]
            torrent["Link"] = (
                config.get("Main", "baseURL")
                + bs4(torrent["Title"], "html.parser").find("a")["href"]
            )
            torrents.append(torrent)
        return torrents

    def contest_leaders(self):
        """Get data on who's winning"""
        LOGGER.debug("Fetching contest leaderboard")
        soup = bs4(session.base_get("contestleaders.php").content, "html.parser")
        ret_array = []
        for cell in (
            soup.find("table", class_="table--panel-like").find("tbody").find_all("tr")
        ):
            ret_array.append(
                (cell.find_all("td")[1].get_text(), cell.find_all("td")[2].get_text())
            )
        return ret_array

    def collage_all(self, coll_id, search_terms={}):
        """Gets all the movies in a collage in one request, only fills torrentid"""
        search_terms["id"] = coll_id
        req = session.base_get("collages.php", params=search_terms)
        soup = bs4(req.content, "html.parser")

        def not_missing_li(tag):
            return tag.has_attr("name") and (tag.name == "li")

        movielist = soup.find(id="collection_movielist").find_all(not_missing_li)
        movies = []
        for page_movie in movielist:
            movieid = page_movie.a["href"].split("id=")[1]
            movies.append(Movie(ID=movieid))
        return movies

    def collage_add(self, coll_id, movieobj):
        """Adds a given movie to a collage, requires password login."""
        search_terms = dict(id=coll_id)
        req = session.base_get("collages.php", params=search_terms)
        soup = bs4(req.content, "html.parser")
        csrf_token = soup.find(id="add_film").find("input")["value"]
        movieobj.load_inferred_data()
        resp = session.base_post(
            "collages.php",
            params=dict(action="add_torrent"),
            data=dict(
                AntiCsrfToken=csrf_token,
                action="add_torrent",
                collageid=coll_id,
                url=movieobj.data["Link"],
            ),
        )
        return resp

    def collage(self, coll_id, search_terms=None):
        """Simplistic representation of a collage, might be split out later"""
        if search_terms is None:
            search_terms = {}
        search_terms["id"] = coll_id
        req = session.base_get("collages.php", params=search_terms)
        movies = []
        for movie in util.snarf_cover_view_data(req.content):
            movie["Torrents"] = []
            for group in movie["GroupingQualities"]:
                movie["Torrents"].extend(group["Torrents"])
            movies.append(Movie(data=movie))
        return movies

    def artist(self, art_id, search_terms=None):
        """Simplistic representation of an artist page, might be split out later"""
        if search_terms is None:
            search_terms = {}
        search_terms["id"] = art_id
        req = session.base_get("artist.php", params=search_terms)
        movies = []
        for movie in util.snarf_cover_view_data(
            req.content, key=b"ungroupedCoverViewJsonData"
        ):
            movie["Torrents"] = []
            for group in movie["GroupingQualities"]:
                movie["Torrents"].extend(group["Torrents"])
            movies.append(Movie(data=movie))
        return movies

    def log(self):
        """Gets the PTP log"""
        soup = bs4(session.base_get("log.php").content, "html.parser")
        ret_array = []
        for message in soup.find("table").find("tbody").find_all("tr"):
            ret_array.append(
                (
                    message.find("span", class_="time")["title"],
                    message.find("span", class_="log__message")
                    .get_text()
                    .lstrip()
                    .encode("UTF-8"),
                )
            )
        return ret_array



# flake8: noqa
"""Exists solely to make 'import ptpapi' possible"""
from ptpapi.api import API
from ptpapi.movie import Movie
from ptpapi.torrent import Torrent
from ptpapi.user import User


def login(**kwargs):
    """A helper function to make it easy to log in"""
    return API(**kwargs)



import logging
import os
import re

import bencode
import humanize

from ptpapi.config import config
from ptpapi.session import TokenSession
from ptpapi.sites.base import BaseSiteAPI


class CGAPI(BaseSiteAPI):
    Name = "CG"

    def __init__(self):
        self.baseURL = "https://cinemageddon.net"
        self.session = TokenSession(3, 0.5)
        self.session.headers.update({"User-Agent": "Wget/1.13.4"})
        super().__init__()

    def login(self, username=None, password=None, passkey=None):
        password = password or config.get("CG", "password")
        username = username or config.get("CG", "username")
        response = self.session.post(
            self.baseURL + "/takelogin.php",
            data={"username": username, "password": password},
        )
        response.raise_for_status()
        if response.text.find('action="takelogin.php"') != -1:
            raise CGAPIException("Failed to log in")

    def search(self, search_args):
        search_string = "&".join(
            ["%s=%s" % (key, value) for (key, value) in search_args.items()]
        )
        soup = self._httpRequest("/browse.php?%s" % search_string)
        return self.getTorrentListInfo(soup)

    def find_ptp_movie(self, movie):
        return self.search({"search": "tt{0}".format(movie["ImdbId"])})

    def bytes_to_site_size(self, byte_num):
        humanized = humanize.naturalsize(byte_num, format="%.2f", binary=True)
        if "KiB" in humanized:
            humanized = humanize.naturalsize(byte_num, format="%d", binary=True)
        return humanized

    def getTorrentListInfo(self, soup):
        if not soup.find("table", class_="torrenttable") or not soup.find(
            "table", class_="torrenttable"
        ).find("tbody"):
            return []
        rows = soup.find("table", class_="torrenttable").find("tbody").find_all("tr")
        retArray = []
        for r in rows:
            data = {}
            data["Title"] = r.find("a", href=re.compile(r"details.php\?id=[0-9]+$"))[
                "title"
            ]
            data["BinaryHumanSize"] = (
                r.find(text=re.compile(r"[0-9]+\.[0-9]+ [kA-Z]B"))
                .replace("B", "iB")
                .replace("k", "K")
            )
            data["Seeders"] = re.match(
                r"([0-9]+)", r.find(title=re.compile("[0-9]+ seeders?"))["title"]
            ).group(1)
            data["ID"] = re.match(
                r"details.php\?id=([0-9]+)$",
                r.find("a", href=re.compile(r"details.php\?id=[0-9]+$"))["href"],
            ).group(1)
            retArray.append(data)
        return retArray

    def download_to_file(self, ID, dest=None):
        logger = logging.getLogger(__name__)
        r = self.session.get(self.baseURL + "/download.php", params={"id": ID})
        r.raise_for_status()
        if not dest:
            name = (
                bencode.bdecode(r.content)["info"]["name"].replace("/", "_")
                + ".torrent"
            )
            dest = os.path.join(config.get("Main", "downloadDirectory"), name)
        logger.debug("Downloading ID {} to {}".format(ID, dest.encode("utf-8")))
        with open(dest, "wb") as fh:
            fh.write(r.content)


class CGAPIException(Exception):
    pass


if __name__ == "__main__":
    cg = CGAPI()
    cg.login()
    print(cg.search({"search": "tt0054650"}))



"""Represent a user"""
import re

from bs4 import BeautifulSoup as bs4  # pylint: disable=import-error

from .movie import Movie
from .session import session
from .util import snarf_cover_view_data


class User:
    """A primitive class to represent a user"""

    def __init__(self, ID):
        # Requires an ID, as searching by name isn't exact on PTP
        self.ID = ID  # pylint: disable=invalid-name

    def __repr__(self):
        return self.__str__()

    def __str__(self):
        return "<ptpapi.User ID %s>" % self.ID

    def bookmarks(self, search_terms=None):
        """Fetch a list of movies the user has bookmarked

        :rtype: array of Movies"""
        search_terms = search_terms or {}
        search_terms.update({"userid": self.ID})
        req = session.base_get("bookmarks.php", params=search_terms)
        movies = []
        for movie in snarf_cover_view_data(req.text):
            movies.append(Movie(data=movie))
        return movies

    def ratings(self):
        """Fetch a list of rated movies

        :rtype: array of tuples with a Movie and a rating out of 100"""
        soup = bs4(
            session.base_get(
                "user.php", params={"id": self.ID, "action": "ratings"}
            ).text,
            "html.parser",
        )
        ratings = []
        for row in soup.find(id="ratings_table").tbody.find_all("tr"):
            movie_id = re.search(r"id=(\d+)", row.find(class_="l_movie")["href"]).group(
                1
            )
            rating = row.find(id="user_rating_%s" % movie_id).text.rstrip("%")
            ratings.append((movie_id, rating))
        return ratings

    def __parse_stat(self, stat_line):
        stat, _, value = stat_line.partition(":")
        stat = stat.title().replace(" ", "").strip()
        value = re.sub(r"\t.*", "", value).replace("[View]", "").strip()
        return stat, value

    def stats(self):
        """
        Return all stats associated with a user

        :rtype: A dictionary of stat names and their values, both in string format.
        """
        soup = bs4(
            session.base_get("user.php", params={"id": self.ID}).text, "html.parser"
        )
        stats = {}
        for li in soup.find("span", text="Stats").parent.parent.find_all("li"):
            stat, value = self.__parse_stat(li.text)
            stats[stat] = value
        for li in soup.find("span", text="Personal").parent.parent.find_all("li"):
            stat, value = self.__parse_stat(li.text)
            if value:
                stats[stat] = value
        for li in soup.find("span", text="Community").parent.parent.find_all("li"):
            stat, value = self.__parse_stat(li.text)
            if stat == "Uploaded":
                match = re.search(r"(.*) \((.*)\)", value)
                stats["UploadedTorrentsWithDeleted"] = match.group(1)
                value = match.group(2)
                stat = "UploadedTorrents"
            elif stat == "Downloaded":
                stat = "DownloadedTorrents"
            elif stat == "SnatchesFromUploads":
                match = re.search(r"(.*) \((.*)\)", value)
                stats["SnatchesFromUploadsWithDeleted"] = match.group(1)
                value = match.group(2)
            elif stat == "AverageSeedTime(Active)":
                stat = "AverageSeedTimeActive"
            stats[stat] = value
        return stats


class CurrentUser(User):
    """Defines some additional methods that only apply to the logged in user."""

    def __init__(self, ID):
        self.new_messages = 0
        super().__init__(ID)

    def archive_container(self, ID):
        """Fetch info about a containers from the archive project

        :returns: A list of dictionaries"""
        torrents = []
        params = {"action": "container", "UserID": self.ID, "containerid": ID}
        soup = bs4(session.base_get("archive.php", params=params).text, "html.parser")
        headers = [
            h.text
            for h in soup.find(class_="table").find("thead").find("tr").find_all("th")
        ]
        rows = soup.find(class_="table").find("tbody").find_all("tr")
        for r in rows:
            # Get as much info as possible from the pages
            row_dict = dict(zip(headers, [f.text for f in r.find_all("td")]))
            # Also add in a Torrent object for creating torrent objects
            if "Torrent Deleted" not in row_dict["Torrent"]:
                row_dict["TorrentId"] = re.search(
                    r"torrentid=([0-9]*)", r.find_all("td")[0].find("a")["href"]
                ).group(1)
        return torrents

    def archive_containers(self):
        """Fetch a list of containers from the archive project

        :returns: A list of dictionaries"""
        containers = []
        soup = bs4(session.base_get("archive.php").text, "html.parser")
        for row in soup.find(class_="table").find("tbody").find_all("tr"):
            cont = {
                "name": row[0].text,
                "link": r[0].find("a")["href"],
                "size": ptpapi.util.human_to_bytes(r[1].text),
                "max_size": ptpapi.util.human_to_bytes(r[2].text),
                "last_fetch": r[3].text,  # TODO: convert this to actual time object
            }
            for field in row:
                if field.find("label"):
                    l = field.find("label")
                    cont[l["title"].lower()] = int(l.text.replace(",", ""))
            containers.append(cont)
        return containers

    def __parse_new_messages(self, soup):
        """Parse the number of messages from a soup of html"""
        msgs = 0
        if soup.find(class_="alert-bar"):
            for alert in soup.find(class_="alert-bar"):
                match = re.search(r"You have (\d+) new message", alert.text)
                if match:
                    msgs = match.group(1)
        return msgs

    def get_new_messages(self):
        """Update the number of messages"""
        soup = bs4(session.base_get("inbox.php").text, "html.parser")
        self.new_messages = self.__parse_new_messages(soup)
        return self.new_messages

    def inbox(self, page=1):
        """Fetch a list of messages from the user's inbox
        Incidentally update the number of messages"""
        soup = bs4(
            session.base_get("inbox.php", params={"page": page}).text, "html.parser"
        )

        self.new_messages = self.__parse_new_messages(soup)

        for row in soup.find(id="messageformtable").tbody.find_all("tr"):
            yield {
                "Subject": row.find_all("td")[1].text.encode("UTF-8").strip(),
                "Sender": row.find_all("td")[2].text,
                "Date": row.find_all("td")[3].span["title"],
                "ID": re.search(r"id=(\d+)", row.find_all("td")[1].a["href"]).group(1),
                "Unread": bool("inbox-message--unread" in row["class"]),
            }

    def inbox_conv(self, conv_id):
        """Get a specific conversation from the inbox"""
        soup = bs4(
            session.base_get(
                "inbox.php", params={"action": "viewconv", "id": conv_id}
            ).text,
            "html.parser",
        )
        messages = []
        for msg in soup.find_all("div", id=re.compile("^message"), class_="forum-post"):
            message = {}
            message["Text"] = msg.find("div", class_="forum-post__body").text.strip()
            username = msg.find("strong").find("a", class_="username")
            if username is None:
                message["User"] = "System"
            else:
                message["User"] = username.text.strip()
            message["Time"] = msg.find("span", class_="time").text.strip()
            messages.append(message)
        return {
            "Subject": soup.find("h2", class_="page__title").text,
            "Message": messages,
        }

    def remove_snatched_bookmarks(self):
        """Remove snatched bookmarks"""
        session.base_post("bookmarks.php", data={"action": "remove_snatched"})

    def remove_seen_bookmarks(self):
        """Remove seen bookmarks"""
        session.base_post("bookmarks.php", data={"action": "remove_seen"})

    def remove_uploaded_bookmarks(self):
        """Remove uploads bookmarks"""
        session.base_post("bookmarks.php", data={"action": "remove_uploaded"})

    def hnr_zip(self):
        """Download the zip file of all HnRs"""
        zip_file = session.base_get("snatchlist.php", params={"action": "hnrzip"})
        if zip_file.headers["Content-Type"] == "application/zip":
            return zip_file
        else:
            return None



import logging

from time import sleep, time

import requests

from urllib3.util.retry import Retry

from .config import config


LOGGER = logging.getLogger(__name__)


class TokenSession(requests.Session):
    """Allows rate-limiting requests to the site"""

    def __init__(self, tokens, fill_rate):
        """tokens is the total tokens in the bucket. fill_rate is the
        rate in tokens/second that the bucket will be refilled."""
        requests.Session.__init__(self)
        self.capacity = float(tokens)
        self._tokens = float(tokens)
        self.consumed_tokens = 0
        self.fill_rate = float(fill_rate)
        self.timestamp = time()

    def consume(self, tokens):
        """Consume tokens from the bucket. Returns True if there were
        sufficient tokens otherwise False."""
        self.get_tokens()
        if tokens <= self.tokens:
            self._tokens -= tokens
            self.consumed_tokens += tokens
            LOGGER.debug("Consuming %i token(s)." % tokens)
        else:
            return False
        return True

    def request(self, *args, **kwargs):
        while not self.consume(1):
            LOGGER.debug("Waiting for token bucket to refill...")
            sleep(1)
        req = requests.Session.request(self, *args, **kwargs)
        return req

    def get_tokens(self):
        if self._tokens < self.capacity:
            now = time()
            delta = self.fill_rate * (now - self.timestamp)
            self._tokens = min(self.capacity, self._tokens + delta)
            self.timestamp = now
        return self._tokens

    tokens = property(get_tokens)

    def base_get(self, url_path, *args, **kwargs):
        return self.get(config.get("Main", "baseURL") + url_path, *args, **kwargs)

    def base_post(self, url_path, *args, **kwargs):
        return self.post(config.get("Main", "baseURL") + url_path, *args, **kwargs)


LOGGER.debug("Initializing token session")
# If you change this and get in trouble, don't blame me
session = TokenSession(3, 0.5)
if config.get("Main", "retry").lower() == "true":
    LOGGER.debug("Setting up automatic retry")
    retry_config = Retry(
        10, connect=4, status=4, backoff_factor=0.5, status_forcelist=[502]
    )
    session.mount("https://", requests.adapters.HTTPAdapter(max_retries=retry_config))
session.headers.update({"User-Agent": "Wget/1.13.4"})



# pylint: disable=invalid-name
"""Hold config values"""
import configparser
import os

from io import StringIO
from pathlib import Path


conf_files = [
    Path(p)
    for p in ["~/.ptpapi.conf", "~/.config/ptpapi.conf", "~/.config/ptpapi/ptpapi.conf"]
]

default = """
[Main]
baseURL=https://passthepopcorn.me/
cookiesFile=~/.ptp.cookies.txt
downloadDirectory=.
filter=
retry=False

[Reseed]
action=hard
findBy=filename,title
"""

env_prefix = "PTPAPI_"
env_keys = {
    "BASEURL": ("Main", "baseURL"),
    "COOKIESFILE": ("Main", "cookiesFile"),
    "DOWNLOADDIRECTORY": ("Main", "downloadDirectory"),
    "FILTER": ("Main", "filter"),
    "RETRY": ("Main", "retry"),
    "APIKEY": ("PTP", "ApiKey"),
    "APIUSER": ("PTP", "ApiUser"),
    "ARCHIVE_CONTAINER_NAME": ("PTP", "archiveContainerName"),
    "ARCHIVE_CONTAINER_SIZE": ("PTP", "archiveContainerSize"),
    "ARCHIVE_MAX_STALLED": ("PTP", "archiveContainerMaxStalled"),
    "RESEED_ACTION": ("Reseed", "action"),
    "RESEED_FINDBY": ("Reseed", "findBy"),
    "PROWLARR_API_KEY": ("Prowlarr", "api_key"),
    "PROWLARR_URL": ("Prowlarr", "url"),
}


config = configparser.ConfigParser()
config.read_file(StringIO(default))
for c in conf_files:
    if c.expanduser().exists():
        config.read(c.expanduser())
        break
else:
    raise ValueError(
        f"Config file not found in any of the following paths: {conf_files!r}"
    )

for key, section in env_keys.items():
    if os.getenv(env_prefix + key) is not None:
        if section[0] not in config.sections():
            config.add_section(section[0])
        config.set(section[0], section[1], os.getenv(env_prefix + key))



from bs4 import BeautifulSoup


class BaseSiteAPI:
    Name = "BS"

    def __init__(self):
        self.baseURL = None
        self.session = None
        self.login()

    def login(self, username=None, password=None, passkey=None):
        raise NotImplementedError

    def download_to_file(self, ID, dest=None):
        raise NotImplementedError

    def find_ptp_movie(self, movie):
        raise NotImplementedError

    def bytes_to_site_size(self, byte_num):
        raise NotImplementedError

    def _httpRequest(self, url, data=None):
        html = self._request(self.baseURL + url, data)
        soup = BeautifulSoup(html, "html.parser")
        return soup

    def _request(self, url, data=None):
        return self.session.get(url, data=data).text

    def _jsonRequest(self, url, data=None):
        return self.session.get(url, data=data).json()



#!/usr/bin/env python
"""The reseed machine runs a scan against prowlarr to find potential
reseeds."""
import argparse
import logging

from urllib.parse import parse_qs, urljoin, urlparse

import requests

import ptpapi

from ptpapi.config import config


def main():
    parser = argparse.ArgumentParser(
        description="Attempt to find torrents to reseed on PTP from other sites"
    )
    parser.add_argument("-i", "--id", help="Only full PTP links for now", nargs="*")
    parser.add_argument(
        "--debug",
        help="Print lots of debugging statements",
        action="store_const",
        dest="loglevel",
        const=logging.DEBUG,
        default=logging.INFO,
    )
    parser.add_argument(
        "-q",
        "--quiet",
        help="Only show error messages",
        action="store_const",
        dest="loglevel",
        const=logging.ERROR,
    )
    parser.add_argument(
        "-l",
        "--limit",
        help="Limit need-for-seed results to N movies",
        default=100,
        type=int,
    )
    parser.add_argument(
        "-s", "--search", help="Allow filtering the need-for-seed results", default=None
    )
    parser.add_argument(
        "-r",
        "--required-remote-seeds",
        help="The number of seeds required on remote torrent",
        default=1,
        type=int,
    )
    parser.add_argument(
        "-m",
        "--min-ptp-seeds",
        help="Set the minimum number of PTP seeds before a reseed attempt will happen",
        default=0,
        type=int,
    )
    args = parser.parse_args()

    logging.basicConfig(level=args.loglevel)
    logger = logging.getLogger("reseed-machine")

    ptp = ptpapi.login()

    if args.id:
        movies = args.id
    else:
        filters = {}
        if args.search:
            for arg in args.search.split(","):
                filters[arg.split("=")[0]] = arg.split("=")[1]
        movies = [t["Link"] for t in ptp.need_for_seed(filters)][: args.limit]

    for i in movies:
        ptp_movie = None
        if "://passthepopcorn.me" in i:
            parsed_url = parse_qs(urlparse(i).query)
            ptp_movie = ptpapi.Movie(ID=parsed_url["id"][0])
            torrent_id = int(parsed_url.get("torrentid", ["0"])[0])

        if ptp_movie is None:
            logger.error("Could not figure out ID '{0}'".format(i))
        else:
            try:
                ptp_movie["ImdbId"]
            except KeyError:
                logger.warning("ImdbId not found from '{0}', skipping".format(i))
                continue
            if ptp_movie["ImdbId"]:
                find_match(ptp_movie, torrent_id)


# Stolen from https://en.wikibooks.org/wiki/Algorithm_Implementation/Strings/Levenshtein_distance#Python
def levenshtein(s1: str, s2: str):
    """Measure the edit distance between two strings"""
    if len(s1) < len(s2):
        return levenshtein(s2, s1)  # pylint: disable=arguments-out-of-order

    if len(s2) == 0:
        return len(s1)

    previous_row = range(len(s2) + 1)
    for i, c1 in enumerate(s1):
        current_row = [i + 1]
        for j, c2 in enumerate(s2):
            insertions = (
                previous_row[j + 1] + 1
            )  # j+1 instead of j since previous_row and current_row are one character longer
            deletions = current_row[j] + 1  # than s2
            substitutions = previous_row[j] + (c1 != c2)
            current_row.append(min(insertions, deletions, substitutions))
        previous_row = current_row

    return previous_row[-1]


def match_results(ptp_result: dict, other_result: dict, title_distance=1) -> dict:
    logger = logging.getLogger("reseed-machine.match")
    percent_diff = 1
    if (
        "imdbid" in other_result
        and other_result["imdbid"]
        and ptp_result["imdbId"] != other_result["imdbId"]
    ):
        logger.debug(
            "%s IMDb ID mismatch: %s %s %s",
            other_result["protocol"],
            other_result["imdbId"],
            other_result["indexer"],
            other_result["title"],
        )
    if other_result["protocol"] == "torrent":
        size_diff = round(
            abs(((other_result["size"] / ptp_result["size"]) - 1) * 100), 2
        )
        if other_result["indexer"] == "PassThePopcorn":
            return {}
        if other_result["seeders"] == 0:
            logger.debug(
                "torrent has no seeders: %s %s",
                other_result["indexer"],
                other_result["title"],
            )
            return {}
        if 0 <= size_diff < percent_diff:
            logger.info(
                "torrent size match: %s %s (%s (%s)), with %.2f%% size diff",
                other_result["indexer"],
                other_result["title"],
                other_result["size"],
                bytes_to_human(other_result["size"]),
                size_diff,
            )
            return other_result
        else:
            logger.debug(
                "torrent size mismatch: %s %s (%s (%s)), with %.2f%% size diff",
                other_result["indexer"],
                other_result["title"],
                other_result["size"],
                bytes_to_human(other_result["size"]),
                size_diff,
            )
    elif other_result["protocol"] == "usenet":
        # Usenet sizes vary wildly based on PAR2 levels,
        # etc, so size comparisons aren't very useful
        if levenshtein(other_result["title"], ptp_result["title"]) <= title_distance:
            logger.info(
                "usenet title match: %s (%s)",
                other_result["indexer"],
                other_result["title"],
            )
            return other_result
        else:
            logger.debug(
                "usenet title mismatch: %s",
                other_result["title"],
            )
        # Also check sortTitle if present
        if "sortTitle" in ptp_result and "sortTitle" in other_result:
            if (
                levenshtein(other_result["sortTitle"], ptp_result["sortTitle"])
                <= title_distance
            ):
                logger.info(
                    "usenet sort title match: %s (%s)",
                    other_result["title"],
                    other_result["sortTitle"],
                )
                return other_result
            else:
                logger.debug(
                    "usenet sort title mismatch: %s (%s)",
                    other_result["title"],
                    other_result["sortTitle"],
                )
    return {}


def bytes_to_human(b: int):
    for count in ["B", "KiB", "MiB", "GiB"]:
        if b < 1024.0:
            return "%3.1f %s" % (b, count)
        b /= 1024.0
    return "%3.1f TiB" % b


def find_match(ptp_movie, torrent_id=0):
    logger = logging.getLogger("reseed-machine.find")
    session = requests.Session()
    session.headers.update({"X-Api-Key": config.get("Prowlarr", "api_key")})
    resp = session.get(
        urljoin(config.get("Prowlarr", "url"), "api/v1/search"),
        params={
            "query": "{ImdbId:" + ptp_movie["ImdbId"] + "}",
            "categories": "2000",
            "type": "movie",
        },
    ).json()

    # Some indexers return completely irrelevant results when the
    # title isn't present.
    ignore_title_indexers = [
        i
        for i in config.get("ReseedMachine", "ignoreTitleResults", fallback="").split(
            ","
        )
        if i
    ]
    for result in resp:
        if result["indexer"] == "PassThePopcorn" and result["seeders"] == 0:
            if torrent_id and f"torrentid={torrent_id}" not in result.get(
                "infoUrl", ""
            ):
                continue
            logger.info(
                "Working torrent %s (size %s (%s), sortTitle '%s')",
                result["title"],
                result["size"],
                bytes_to_human(int(result["size"])),
                result["sortTitle"],
            )
            download = {}
            for other_result in resp:
                download = match_results(result, other_result)
                if download:
                    break
            # If no match found, search again by release title
            if not download:
                release_title_resp = session.get(
                    urljoin(config.get("Prowlarr", "url"), "api/v1/search"),
                    params={
                        "query": result["title"],
                        "type": "search",
                        "limit": 100,
                    },
                ).json()
                for release_result in release_title_resp:
                    if release_result["indexer"] not in ignore_title_indexers:
                        download = match_results(result, release_result)
                        if download:
                            break

            if download:
                logger.info(
                    "Downloading %s (%s) from %s",
                    download["title"],
                    download["infoUrl"],
                    download["indexer"],
                )
                r = session.post(
                    urljoin(config.get("Prowlarr", "url"), "api/v1/search"),
                    json={
                        "guid": download["guid"],
                        "indexerId": download["indexerId"],
                    },
                )
                r.raise_for_status()


if __name__ == "__main__":
    main()



import html
import json
import math
import re
import urllib

from bs4 import BeautifulSoup as bs4

from ptpapi.error import PTPAPIException


def raise_for_cloudflare(text):
    """Raises an exception if a CloudFlare error page is detected

    :param text: a raw html string"""
    soup = bs4(text, "html.parser")
    if soup.find(class_="cf-error-overview") is not None:
        msg = "-".join(soup.find(class_="cf-error-overview").get_text().splitlines())
        raise PTPAPIException("Encountered Cloudflare error page: ", msg)


def sizeof_fmt(num, suffix="B"):
    for unit in ["", "Ki", "Mi", "Gi", "Ti", "Pi", "Ei", "Zi"]:
        if abs(num) < 1024.0:
            return "%3.1f%s%s" % (num, unit, suffix)
        num /= 1024.0
    return "%.1f%s%s" % (num, "Yi", suffix)


# Adapted from https://gist.github.com/leepro/9694638

SYMBOLS = {
    "customary": ("B", "K", "M", "G", "T", "P", "E", "Z", "Y"),
    "customary_ext": (
        "byte",
        "kilo",
        "mega",
        "giga",
        "tera",
        "peta",
        "exa",
        "zetta",
        "iotta",
    ),
    "iec": ("Bi", "Ki", "Mi", "Gi", "Ti", "Pi", "Ei", "Zi", "Yi"),
    "iec_b": ("BiB", "KiB", "MiB", "GiB", "TiB", "PiB", "EiB", "ZiB", "YiB"),
    "iec_ext": ("byte", "kibi", "mebi", "gibi", "tebi", "pebi", "exbi", "zebi", "yobi"),
}


def human_to_bytes(s, case_sensitive=True):
    """
    Attempts to guess the string format based on default symbols
    set and return the corresponding bytes as an integer.
    When unable to recognize the format ValueError is raised.

      >>> human2bytes('0 B')
      0
      >>> human2bytes('1 K')
      1024
      >>> human2bytes('1 M')
      1048576
      >>> human2bytes('1 Gi')
      1073741824
      >>> human2bytes('1 tera')
      1099511627776

      >>> human2bytes('0.5kilo')
      512
      >>> human2bytes('0.1  byte')
      0
      >>> human2bytes('1 k')  # k is an alias for K
      1024
      >>> human2bytes('12 foo')
      Traceback (most recent call last):
          ...
      ValueError: can't interpret '12 foo'
    """
    try:
        return int(s)
    except ValueError:
        pass
    s = s.replace(",", "")
    init = s
    num = ""
    while s and s[0:1].isdigit() or s[0:1] == ".":
        num += s[0]
        s = s[1:]
    num = float(num)
    letter = s.strip()
    for _, sset in SYMBOLS.items():
        if letter in sset:
            break
        elif not case_sensitive and letter.lower() in [s.lower() for s in sset]:
            break
    else:
        if letter == "k":
            # treat 'k' as an alias for 'K' as per: http://goo.gl/kTQMs
            sset = SYMBOLS["customary"]
            letter = letter.upper()
        else:
            raise ValueError("can't interpret %r" % init)
    prefix = {sset[0]: 1}
    for i, sval in enumerate(sset[1:]):
        if case_sensitive:
            prefix[sval] = 1 << (i + 1) * 10
        else:
            prefix[sval.lower()] = 1 << (i + 1) * 10
            letter = letter.lower()
    return int(num * prefix[letter])


def snarf_cover_view_data(text, key=rb"coverViewJsonData\[\s*\d+\s*\]"):
    """Grab cover view data directly from an html source
    and parse out any relevant infomation we can

    :param text: a raw html string
    :rtype: a dictionary of movie data"""
    data = []
    for json_data in re.finditer(key + rb"\s*=\s*({.*});", text, flags=re.DOTALL):
        data.extend(json.loads(json_data.group(1).decode())["Movies"])
        for movie in data:
            movie["Title"] = html.unescape(movie["Title"])
            movie["Torrents"] = []
            for group in movie["GroupingQualities"]:
                for torrent in group["Torrents"]:
                    soup = bs4(torrent["Title"], "html.parser")
                    if len(soup.a.text.split("/")) < 4:
                        continue
                    (
                        torrent["Codec"],
                        torrent["Container"],
                        torrent["Source"],
                        torrent["Resolution"],
                    ) = [item.strip() for item in soup.a.text.split("/")[0:4]]
                    if soup.contents[0].string is not None:
                        torrent["GoldenPopcorn"] = (
                            soup.contents[0].string.strip(" ") == "\u10047"
                        )  # 10047 = Unicode GP symbol pylint: disable=line-too-long
                    if "title" not in soup.a:
                        continue
                    torrent["ReleaseName"] = soup.a["title"].split("\n")[-1]
                    match = re.search(
                        r"torrents.php\?id=(\d+)&torrentid=(\d+)", soup.a["href"]
                    )
                    torrent["Id"] = match.group(2)
                    movie["Torrents"].append(torrent)
    return data


def find_page_range(text) -> int:
    """From a full HTML page, try to find the number of available
    pages."""
    # Try loading as a big JSON
    try:
        data = json.loads(text)
        return math.ceil(int(data["TotalResults"]) / len(data["Movies"]))
    except json.decoder.JSONDecodeError:
        pass
    # Try parsing pagination infromation from HTML
    soup = bs4(text, "html.parser")
    url = soup.select("a.pagination__link--last")[0]["href"]
    qs = urllib.parse.parse_qs(urllib.parse.urlparse(url).query)
    return int(qs["page"][0])



"""Represent a single torrent object"""
import html
import logging
import os
import re

from pathlib import Path
from urllib.parse import parse_qs, urlparse

import humanize

from bs4 import BeautifulSoup as bs4

from ptpapi import movie
from ptpapi.config import config
from ptpapi.error import PTPAPIException
from ptpapi.session import session


LOGGER = logging.getLogger(__name__)


class Torrent:
    """Represent a single torrent"""

    def __init__(self, ID=None, data=None):
        self.key_finder = {
            "movie_json": [
                "Checked",
                "Codec",
                "Container",
                "GoldenPopcorn",
                "GroupId",
                "InfoHash",
                "Leechers",
                "Quality",
                "ReleaseGroup",
                "ReleaseName",
                "RemasterTitle",
                "Resolution",
                "Scene",
                "Seeders",
                "Size",
                "Snatched",
                "Source",
                "UploadTime",
            ],
            "torrent_json": ["Description", "Nfo"],
            "movie_html": ["Filelist"],
            "inferred": ["Link", "Id", "HumanSize"],
            "torrent_description": ["BBCodeDescription"],
            "parent": [
                "Movie"  # Would be 'inferred' if it didn't have a chance to trigger a request
            ],
        }

        if data:
            self.data = data
            if "Id" in data:
                self.ID = data["Id"]  # pylint: disable=invalid-name
            elif "TorrentId" in data:
                self.ID = data["TorrentId"]
            else:
                raise PTPAPIException("Could not find torrent ID in data")
        elif ID:
            self.ID = ID
            self.data = {"Id": ID}
        else:
            raise PTPAPIException("Not enough information to intialize torrent")

    def __repr__(self):
        return "<ptpapi.Torrent ID %s>" % self.ID

    def __str__(self):
        return "<ptpapi.Torrent ID %s>" % self.ID

    def __nonzero__(self):
        return self.ID is not None

    def __getitem__(self, name):
        if name not in self.data or self.data[name] is None:
            for k, value in self.key_finder.items():
                if name in value:
                    getattr(self, "load_%s_data" % k)()
        return self.data[name]

    def __setitem__(self, key, value):
        self.data[key] = value

    def items(self):
        """Passthru for underlying dict"""
        return self.data.items()

    def keys(self):
        """Passthru for underlying dict"""
        return self.data.keys()

    def load_torrent_description_data(self):
        self.data["BBCodeDescription"] = html.unescape(
            session.base_get(
                "torrents.php", params={"id": self.ID, "action": "get_description"}
            ).text
        )

    def load_movie_html_data(self):
        """Get data from the parent movie's JSON data"""
        if "GroupId" not in self.data or not self.data["GroupId"]:
            movie_url = session.base_get(
                "torrents.php", params={"torrentid": self.ID}
            ).url
            self.data["GroupId"] = parse_qs(urlparse(movie_url).query)["id"][0]
        soup = bs4(
            session.base_get(
                "torrents.php", params={"id": self.data["GroupId"], "json": 0}
            ).content,
            "html.parser",
        )
        filediv = soup.find("div", id="files_%s" % self.ID)
        self.data["Filelist"] = {}
        for elem in filediv.find("tbody").find_all("tr"):
            bytesize = (
                elem("td")[1]("span")[0]["title"].replace(",", "").replace(" bytes", "")
            )
            self.data["Filelist"][elem("td")[0].string] = bytesize
        # Check if trumpable
        if soup.find("trumpable_%s" % self.ID):
            self.data["Trumpable"] = [
                s.get_text() for s in soup.find("trumpable_%s" % self.ID).find("span")
            ]
        else:
            self.data["Trumpable"] = []

    def load_movie_json_data(self):
        """Load data from the movie page"""
        LOGGER.debug("Loading Torrent data from movie JSON page.")
        if "GroupId" not in self.data or not self.data["GroupId"]:
            movie_url = session.base_get(
                "torrents.php", params={"torrentid": self.ID}
            ).url
            self.data["GroupId"] = re.search(r"\?id=(\d+)", movie_url).group(1)
        movie_data = session.base_get(
            "torrents.php",
            params={"torrentid": self.ID, "id": self.data["GroupId"], "json": "1"},
        ).json()
        for tor in movie_data["Torrents"]:
            if int(tor["Id"]) == int(self.ID):
                # Fill in any optional fields
                for key in ["RemasterTitle"]:
                    if key not in self.data:
                        self.data[key] = ""
                self.data.update(tor)
                break

    def load_inferred_data(self):
        self.data["Id"] = self.ID
        self.data["Link"] = "https://passthepopcorn.me/torrents.php?torrentid=" + str(
            self.ID
        )
        self.data["HumanSize"] = humanize.naturalsize(
            int(self.data["Size"]), binary=True
        )

    def load_parent_data(self):
        self.data["Movie"] = movie.Movie(ID=self["GroupId"])

    def load_torrent_json_data(self):
        """Load torrent data from a JSON call"""
        LOGGER.debug("Loading Torrent data from torrent JSON page.")
        if "GroupId" not in self.data or not self.data["GroupId"]:
            movie_url = session.base_get(
                "torrents.php", params={"torrentid": self.ID}
            ).url
            self.data["GroupId"] = re.search(r"\?id=(\d+)", movie_url).group(1)
        self.data.update(
            session.base_get(
                "torrents.php",
                params={
                    "action": "description",
                    "id": self.data["GroupId"],
                    "torrentid": self.ID,
                },
            ).json()
        )
        if "Nfo" in self.data and self.data["Nfo"]:
            self.data["Nfo"] = html.unescape(self.data["Nfo"])

    def download(self, params=None):
        """Download the torrent contents"""
        if params is None:
            params = {}
        req_params = params.copy()
        req_params.update({"action": "download", "id": self.ID})
        req = session.base_get("torrents.php", params=req_params)
        return req.content

    def download_to_dir(self, dest=None, params=None):
        """Convenience method to download directly to a directory"""
        if params is None:
            params = {}
        req_params = params.copy()
        req_params.update({"action": "download", "id": self.ID})
        req = session.base_get("torrents.php", params=req_params)
        if not dest:
            dest = Path(config.get("Main", "downloadDirectory"))
        name = re.search(r'filename="(.*)"', req.headers["Content-Disposition"]).group(
            1
        )
        dest = Path(dest, name)
        with dest.open("wb") as fileh:
            fileh.write(req.content)
        return dest



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['ptpapi', 'ptpapi.scripts', 'ptpapi.sites']

package_data = \
{'': ['*']}

install_requires = \
['Tempita>=0.5.2,<0.6.0',
 'beautifulsoup4>=4.10.0,<5.0.0',
 'bencode.py>=4.0.0,<5.0.0',
 'guessit>=3.4.3,<4.0.0',
 'humanize>=4.0.0,<5.0.0',
 'libtc>=1.3.1,<2.0.0',
 'pyrosimple>=2.7.0,<3.0.0',
 'requests>=2.27.1,<3.0.0']

entry_points = \
{'console_scripts': ['ptp = ptpapi.scripts.ptp:main',
                     'ptp-reseed = ptpapi.scripts.ptp_reseed:main',
                     'ptp-reseed-machine = '
                     'ptpapi.scripts.ptp_reseed_machine:main']}

setup_kwargs = {
    'name': 'ptpapi',
    'version': '0.9.1',
    'description': 'A small API for a mildly popular movie site',
    'long_description': '# PTPAPI\n\n[![PyPI](https://img.shields.io/pypi/v/ptpapi)](https://pypi.org/project/ptpapi/)\n![PyPI - Python Version](https://img.shields.io/pypi/pyversions/ptpapi)\n\nA small API for a mildly popular movie site. The goal was to be able\nto collect as much information in as few network requests as possible.\n\n## Dependencies\n\n* Python 3.7+\n* pip\n\n## Installation\n\nUse of a\n[virtualenv](https://virtualenv.readthedocs.org/en/latest/userguide.html#usage)\nor [pipx](https://pypa.github.io/pipx/) is optional, but highly\nrecommended.\n\n`pip install ptpapi`\n\n## Configuration\n\nOpen the file `~/.ptpapi.conf` for editing, and make sure it looks like the following:\n\n```ini\n[Main]\n\n[PTP]\nApiUser=<ApiUser>\nApiKey=<ApiKey>\n```\n\nBoth values can be found in the "Security" section of your\nprofile. This is only the minimum required configuration. See\n`ptpapi.conf.example` for a full-futured config file with comments.\n\n## Usage\n\nThe three CLI commands are `ptp`, `ptp-reseed`, and `ptp-bookmarks`\n\n### `ptp`\n\nThis is a generally utility to do various things inside PTP. As of\nright now it can download files, search the site for movies, and list\nmessage in your inbox.\n\nSee `ptp help` for more information.\n\n#### `ptp inbox`\n\nA small utility to read messages in your inbox. No reply capability currently.\n\n#### `ptp download`\n\nAn alias for `ptp-search -d`\n\n#### `ptp search`\n\nThis subcommand lets you search the site for movies. It can take movie\nand permalinks, as well as search by arbitrary parameters, and the\n`-d` flag allows for downloading matching torrents. For instance:\n- `ptp search year=1980-2000 taglist=sci.fi`\n- `ptp search "Star Wars"`.\n\nIt can also accept URLs for torrents and collages:\n- `ptp search "https://passthepopcorn.me/torrents.php?id=68148"`\n- `ptp search "https://passthepopcorn.me/collages.php?id=2438"`\n\nand regular search URLs:\n- `ptp search "https://passthepopcorn.me/torrents.php?action=advanced&year=1980-2000&taglist=action"`.\n\nAs a general rule of thumb anything supported by the advanced site\nsearch will work with `ptp search`, e.g. searching\n`https://passthepopcorn.me/torrents.php?action=advanced&taglist=comedy&format=x264&media=Blu-ray&resolution=1080p&scene=1`\nis the same as `ptp search taglist=comedy format=x264 media=Blu-ray\nresolution=1080p scene=1`.\n\nTo work with multiple pages of results, use the `--pages <num>` flag.\n\nThere are a couple aliases to make life easier:\n\n* `genre`, `genres`, `tags` -> `taglist`\n* `name` -> `searchstr`\n* `bookmarks` -> Search only your bookmarks\n\nIn addition, [Tempita](http://pythonpaste.org/tempita/) can be used\nfor custom formatting. For instance, `ptp search --movie-format=""\n--torrent-format="{{UploadTime}} - {{ReleaseName}}" year=1980-2000\ntaglist=sci.fi grouping=no`.\n\nUsing the `-d` flag will download one torrent from each of the matched\ntorrents (deciding which one to download is done via\n[filters](#filters)) to the\n[downloadDirectory](ptpapi.conf.example#L9).\n\nThe `-p/--pages [int]` option can be used to scrape multiple pages at\nonce. N.B.: If any `page` parameter is in the original search query,\npaging will start from that page.\n\n#### `ptp fields`\n\nSimply list fields that can be used for the `ptp search` formatting.\n\n### `ptp-reseed`\n\nThis script automatically matches up files to movies on PTP. It\'s most\nbasic usage is `ptp-reseed <file path>`. This will search PTP for any\nmovies matching that filename, and if it finds a match, will\nautomatically download the torrent and add it to rtorrent. It can do\nsome basic file manipulation if it finds a close enough match.\n\nFor instance, if you have the file `Movie.2000.mkv`, and the torrent\ncontains `Movie (2000)/Movie.2000.mkv`, the script will try to\nautomatically create the folder `Movie (2000)` and hard link the file\ninside of it before attempting to seed it.\n\nSee `ptp-reseed -h` and `ptpapi.conf.example` for more information and\nconfiguration options.\n\n#### guessit\n\nBy default the script looks for exact matches against file names and\nsizes. If you\'d like the name matching to be less strict, you can\ninstall the guessit library (`pip install \'guessit>=3\'`), and if the\nfilename search fails, the script will attempt to parse the movie name\nout of the file with guessit.\n\n### `ptp-reseed-machine`\n\nThis tool is meant to complement `ptp-reseed`, by taking using\n[Prowlarr](https://github.com/Prowlarr/Prowlarr) to find and download\npeotential reseeds from any supported site.\n\nTo get it set up, first [install\nProwlarr](https://wiki.servarr.com/prowlarr/installation). Be sure\nyour instance (or any of the *arrs) isn\'t exposed to the internet!\nFrom, simply use the UI to add any trackers/indexers you\'d like to\nsearch, as well as any downloaders. Then, add the following config to\n`~/.ptpapi.conf`:\n\n```ini\n[Prowlarr]\nurl=http://YOUR_PROWLER_HOSTNAME_OR_IP/\napi_key=YOUR_API_KEY\n```\n\nIf everything thing is all setup, running `ptp-reseed-machine` will\nscrape the first page of needforseed.php and attempt to download any\npotential matches. See `--help` for passing additional parameters or\ndifferent search targets.\n\nAfter a download has been triggered, you can then use `ptp-reseed`\nwith your download client of choice to automatically reseed the path\ninto a client. Here\'s a simple example of a post script for sabnzbd:\n\n```bash\n#!/bin/bash\nif [[ "$SAB_PP_STATUS" -eq 0 ]]; then\n  ls *.iso *.img 2>/dev/null | xargs -r 7z x\n  ptp-reseed "$SAB_COMPLETE_DIR"\nfi\n```\n\nor for a `rtorrent.rc`:\n```ini\nmethod.set_key = event.download.finished, ptp_reseed, "execute.nothrow.bg={ptp-reseed,$d.base_path=}"\n```\n\n## Concepts\n\n### Filters\n\nFilters were designed as a way to take a full movie group, and narrow\nit down to a single torrent. A filter consists of multiple\nsub-filters, where the first sub-filter to match will download the\ntorrent, and if not, the next sub-filter will be checked. If none of\nthe sub-filters match, no download will occur. Filters are separate\nfrom the actual search parameters sent to the site\n\nThe full list of possible values for picking encodes is:\n* `GP` or `Scene`\n* `576p` or `720p` or `1080p`\n* `XviD` or `x264`\n* `HD` or `SD`\n* `remux` or `not-remux`\n* `seeded` - the number of seeds is greater than 0 (deprecated, use `seeders>0`)\n* `not-trumpable` - ignore any trumpable torrents\n* `unseen` - ignores all torrents if you\'ve marked the movie as seen or rated it\n* `unsnatched` - ignore all torrents unless you\'ve never snatched one before (note that seeding counts as "snatched", but leeching doesn\'t)\nThere are also values that allow for simple comparisons, e.g. `size>1400M`.\n* `seeders`\n* `size`\n\nNote that it\'s possible to have two incompatible values, e.g. `GP` and\n`Scene`, but this simply means the sub-filter won\'t ever match a\ntorrent, and will always be skipped over.\n\nThe possible values for sorting are:\n* `most recent` (the default if none are specified)\n* `smallest`\n* `most seeders`\n* `largest`\n\n#### Examples\n\nFor instance, the filter `smallest GP,720p scene,largest` would\nattempt to download the smallest GP. If there are no GPs, it will try\nto find a 720p scene encode. If it can\'t find either of those, it will\njust pick the largest torrent available.\n\nAs another example, if you wanted to filter for encodes that are less\nthan 200MiB with only one seeder, you could use `seeders=1 size<200M`.\n\n## Notes\n\nI did this mostly for fun and to serve my limited needs, which is why\nit\'s not as polished as it could be, and will probably change\nfrequently.  Pull requests are welcomed.\n\n### Deprecated Configuration\n\nThe new ApiUser/ApiKey system is preferred, however if you find bugs\nor limitations, the old cookie-based method can be used:\n\nOpen the file `~/.ptpapi.conf` for editing, and make sure it looks\nlike the following:\n\n```ini\n[Main]\n\n[PTP]\nusername=<username>\npassword=<password>\npasskey=<passkey>\n```\n',
    'author': 'kannibalox',
    'author_email': 'kannibalox@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/kannibalox/PTPAPI',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7.2,<4.0',
}


setup(**setup_kwargs)
